﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: (any)

DESCRIPTION:
    
This is a script that iterates over all open documents
and reports their filenames. It does not manipulate 
or change any files. It is best to run if you have
several documents open.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the first open document in the session with the
//"FirstOpenDoc" property of the session. There is no
//particular order to the internal "stack" of open documents,
//so you will never know which one you will get. Don't make
//the mistake of thinking it will always be the first one you opened,
//or the one in the left-most tab in the FM window.
var doc = app.FirstOpenDoc;

//Initialize this variable. We'll use this as a counter later... read on.
var openDocCount = 0;

//At this point, we either have a document object,
//or an invalid object if there are no open documents.
//We'll use a "while" loop to iterate over all active documents,
//testing with the "ObjectValid()" method each time we try to 
//get another document.  This is a very typical way to iterate over
//lists of objects with ExtendScript, lists which FM keeps internally
//and exposes through these properties. You just get the first one,
//test it, get the next one, test it, etc. etc. until you end up with an
//invalid object. That means you've gone past the last one.

//This model is how you would iterate over formats (paragraph, table, xref, etc.),
//cross-references, components in a book, etc. If you go further with 
//ExtendScript, you are likely to see it countless times.

//So anyway, this loop will execute repeatedly as long as the "doc" object
//is not null and is a valid object.
while(doc != null && 
          doc.ObjectValid() == true) 
{
    //One little fancy thing. For reporting purposes, we'll keep a counter of the
    //number of documents we find.
    openDocCount = openDocCount + 1;
    
    //Now, let's prompt with the filename. One thing we will do here
    //is use the confirm prompt so we can capture a Yes/No
    //response from the user. If the user clicks No, we will 
    //set the doc object to null so the loop will exit.
    var docPath = doc.Name;
    if(docPath == "") docPath = "(No path found, might be a hidden or unsaved document!)";
    
    var promptMessage = "Found open document #" + openDocCount + ":\n\n  " +
        docPath +
        "\n\nWould you like to look for the next one?";
    
    var answer = confirm(promptMessage);
    
    //If the user clicked No...
    if(answer == false)
    {
        alert("Script aborted.");
        doc = null;
    }
    
    //Otherwise, for Yes....
    else
    {
        //A document object has a "NextOpenDocInSession" property
        //which is the next open document in the internal list. Every
        //document knows where it is in the list and what is the next document.
        
        //Note that:
        
        // - This property belongs to the document object, not the session.
        //    It's logical to ask the session for the first doc in the list, but not the
        //    next doc, because it doesn't store any kind of state of what a script
        //    is asking for. The child object (that is, the list item or doc in this case)
        //    knows where it is, so we ask it to give us the next one. It is very important
        //    to understand this concept when iterating over object lists. It always works 
        //    like this.
        
        // - We are resetting the "doc" variable with the return from the property
        //    call. In a programming language like this, the processor is
        //    smart enough to retrieve what we want from the original doc 
        //    object, before it gets overwritten by the new object.
        //    Again, this is very common and important to understand.
        doc = doc.NextOpenDocInSession;
        
        //if the object is not valid, we know the script will exit.
        if(doc.ObjectValid() == false)
        {
            alert("All open documents have been found.\n\nTotal: " + openDocCount);
        }
    }
}

alert("Script complete!");

