﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: (any)

DESCRIPTION:
    
This script reports various properties of the
active document; that is, properties of the
document object. It is intended as an introduction
to the usage of a small subset of the wide
variety of properties that a document object
owns. See the scripting guide for more information.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the currently-active document
var doc = app.ActiveDoc;

//If we have an active document, let's continue.
if(doc.ObjectValid() == true) 
{
    //Introductory prompt with the filename.
    alert("This script will report various things about the active file:\n\n"
        + doc.Name);
        
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////
        
    //Let's count how many paragraphs in the main flow.
    //Here's how to get the first one... it's a drill-down process. First,
    //we have to get the main flow in the doc, which is a property of the
    //document. Then we get the first text frame in the flow, which is a property
    //of the flow. Then we can get the first paragraph in the frame, which is a 
    //property of the frame. This cascading technique to drill through ownership
    //is common in ExtendScript. You could write separate statements for
    //each step, or use the "dot" shortcut as shown below. It is often much
    //cleaner to use the latter.
    var pgf = doc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;
    
    //Let's initiatize this variable to hold the count.
    var counter = 0;
    
    //Now step through each paragraph, incrementing the counter.
    while(pgf.ObjectValid())
    {
        counter = counter + 1;
        pgf = pgf.NextPgfInFlow;
    }

    //and report
    alert("The document has " + counter + " paragraph(s) in the main flow.");
    
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Let's count the number of paragraph formats. The document
    //owns the format objects.
    var format = doc.FirstPgfFmtInDoc
    
    //Reinitialize our counter
    counter = 0;
    
    //Now step through each paragraph, incrementing the counter.
    //We'll do something a little more fancy this time... we'll remember 
    //the first and last formats. These will not necessarily be in 
    //alphabetical order.
    var firstFmtName = "";
    var lastFmtName = "";
    
    while(format.ObjectValid())
    {
        //increment the counter
        counter = counter + 1;
        
        //If we are at the first one in the list, let's store 
        //the name
        if(counter == 1) firstFmtName = format.Name;
        
        //Let's always record the current format name as
        //the last name. When this loop aborts, whatever
        //we were last left with will represent the last format
        //name detected.
        //This architecture does not represent the best
        //way to do this from a programming perspective,
        //as it is more "expensive" (processor intensive)
        //than some other ways we could do it. But it is simple
        //and adequate for this sample.
        lastFmtName = format.Name;
        
        //And, get the next format in the catalog. When we've
        //gone past the last one, this call will return an invalid
        //object and the while loop will abort.
        format = format.NextPgfFmtInDoc;
    }

    //At this point, we have the count, the first format name, and the name of the
    //last valid format found. Let's report.
    alert("The document has " 
        + counter
        + " paragraph formats in the catalog. The first one detected is:\n\n  "
        + firstFmtName
        + "\n\nThe last one detected is:\n\n  "
        + lastFmtName
        + "\n\nNote that this may not represent alphabetical order!");
    
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Let's count the number of character formats. This time,
    //we'll report the first one in alphabetical order. We'll
    //need to use a little JavaScript footwork to do this.
    //owns the format objects.
    var format = doc.FirstCharFmtInDoc
    
    //Reinitialize our counter
    counter = 0;
    
    //Initialize this to hold our format name
    var fmtName = "";
    
    while(format.ObjectValid())
    {
        //increment the counter
        counter = counter + 1;
        
        //If our currently-stored name is zilch or it is
        //"less than" the currently-stored name, let's
        //reset our variable
        if(fmtName == "" ||
            format.Name.toLowerCase() < fmtName.toLowerCase())
                fmtName = format.Name;
        
        //And, get the next format in the catalog. 
        format = format.NextCharFmtInDoc;
    }

    //At this point, we have the count, the first format name, and the name of the
    //last valid format found. Let's report.
    alert("The document has " 
        + counter
        + " character formats in the catalog. "
        + "The first one in alphabetical order is:\n\n  "
        + fmtName);

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Let's report the color assigned for use with change bars.
    //The ChangeBarColor property returns a Color object, 
    //from which we query the color name property.
    alert("The color assigned to use for change bars is:\n\n  "
        + doc.ChangeBarColor.Name);

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Let's report if "smart quotes" are on or off
    if(doc.SmartQuotes == true)
        alert("Smart quotes are on!");
    else alert("Smart quotes are off.");

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Let's report if the main flow is structured or not. We
    //can determine this by looking for a valid highest-level
    //element in the flow.
    if(doc.MainFlowInDoc.HighestLevelElement.ObjectValid())
        alert("The main flow is structured!");
    else alert("The main flow is unstructured.");

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Report the zoom. The property returns a zoom value
    //in the form of a "metric", a special data type that can 
    //be very confusing. For the sake of this tutorial, just
    //accept that 1% = 655.36
    alert("The current zoom is:\n\n  "
        + doc.Zoom / 655.36 
        + "%");

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Report the body page height. Again, this
    //property deals with metrics. For the sake of this tutorial,
    //just accept that 1 inch = 4718592 "metrics" and 
    //1 centimeter = 1857713
    alert("The height of body pages is:\n\n  "
        + doc.PageHeight / 4718592 
        + " inches (" 
        + doc.PageHeight / 1857713
        + " centimeters)");

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////

    //Now just for fun, let's write something to the status line of the
    //document window, that message area down at the bottom left.
    doc.StatusLine = "HEY!! You just ran a script on me!";
    
    alert("Script complete! Hint - look at the status bar "
        + "at the bottom of the document window!");
}

else 
    alert("No active document. Cannot continue.");

