﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:

  SampleFile_UNSTRUCTURED.fm

DESCRIPTION:    

This is a script that iterates over all paragraphs
in the main flow and reports their first 50 characters
and the paragraph format in use. Note that it will
not look inside of tables.

The sample file listed is a good candidate on which
to run the script, but any file with a main flow will 
probably work fine.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Let's declare these here, which will
//be used more later.
var pgfText = "";
var msg = "";
var totalPgfs = 0;

//Time to start the main operation. Get the active document first.
var doc = app.ActiveDoc;

//If we have an active document, let's
//do the requested task.
if(doc.ObjectValid() == true)
{
    //Let's initialize our message string.
    msg = "Paragraph report for: \n\n" +
        doc.Name + "\n\n";
        
    //We'll start by getting the first paragraph in the main flow,
    //then iterate over each paragraph, checking the name of the
    //last format applied, then reapplying the updated format as applicable.
    //To get the first paragraph, notice how we drill down through object
    //properties and "ownership." The doc owns the main flow, the flow owns
    //its first text frame, and a text frame owns its first paragraph.
    var pgf = doc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;
    
    //Run this loop until we run out of valid paragraphs
    while(pgf.ObjectValid() == true)
    {
        //We can increment our paragraph counter now.
        totalPgfs = totalPgfs + 1;
        
        //Let's get the paragraph format first, because it is
        //the easiest part. The name of the most recently-applied
        //format is stored as the Name property of the paragraph.
        var formatName = pgf.Name;
        
        //Now, to get the text is a little bit more tricky. First, we have to establish
        //a "text range," which has some similarity to making a selection
        //in the document window. A text range consists of a starting point
        //and an ending point, much like a selection in a document. A
        //point is identified by the parent paragraph and the character
        //offset from the beginning. In this case, we want to select the
        //whole paragraph. The FV_OBJ_END_OFFSET is a special
        //constant that allows us to indicate the end without actually
        //knowing how many characters it is until the end.
        
        //Text ranges can be tough to get your mind around. Don't expect
        //to understand them immediately.  There is no document window,
        //mouse, or keyboard available when you need to do something
        //from a script. So, this is the methodology that scripts use. Allow
        //some time and experimentation to get the hang of it.
        var textRange = new TextRange();
        textRange.beg.obj = pgf;
        textRange.beg.offset = 0;
        textRange.end.obj = pgf;
        textRange.end.offset = Constants.FV_OBJ_END_OFFSET;

        //Now, once you have a text range, you can call this special
        //document method to get the text from it. Text is retrieved
        //as a "text items" structure, something that is even more
        //complicated than text ranges. Comments here will be minimal.
        //If you are new to scripting, this will take you some time to
        //get comfortable with. Although you might think it should
        //be otherwise, working with text is one of the most difficult
        //things to do with a script.
        var textItems = doc.GetTextForRange(textRange, Constants.FTI_String);
        
        //Now, let's isolate the text into a single variable for convenience. It's
        //stored in the text items, but we'll do this to make the rest of the
        //script more intuitive. One thing we'll need to do is make
        //sure that there actually is any text. We can test the length of the
        //text items structure to know for sure.
        
        //So, if there was any text retrieved...
        if(textItems.length > 0)
        {
            //Again, if you are new to FM scripting, this will
            //look very cryptic to you. Expect some study
            //and experimentation time to understand it better.
            pgfText = textItems[0].sdata;
            
            //Let's truncate the string down to 50 characters if
            //necessary, for the sake of the message box
            if(pgfText.length > 50) 
                pgfText = pgfText.substring(0, 50);
        }
        //Else, if there was no text, let's plan to report that
        else pgfText = "(No text!)";
        
        //Now, we have all the items we want to report. Let's
        //augment the message string. One thing... let's
        //limit this to the first 25 paragraphs, because otherwise
        //who knows how big this message box could get.
        if(totalPgfs < 26)
        {
            msg = msg + "Pgf #" + totalPgfs +
                ": \"" + pgfText + "\"  (Format: " +
                formatName + ")\n";
        }
    
        //Get the next paragraph in the flow. The current
        //paragraph knows what the next one is.
        pgf = pgf.NextPgfInFlow;
    }

    //Now we are all done looking through paragraphs.
    //Let's do the final touchups on the alert box message
    msg = msg + "\n" + totalPgfs + " paragraphs found in main flow."
    
    //If we found more than 25 pgfs, let's add an additional note 
    if(totalPgfs > 25) 
        msg = msg + " Note that only the first " +
            "25 paragraphs are reported above.";
    
    //And report!
    alert(msg);

}

//Else, if we did not find an active document, let's
//warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");
