﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_UNSTRUCTURED.fm

DESCRIPTION:

This is a script that changes the font size
of a paragraph format in the active document.

As written, it will change the font size
of the Body format to 24 points. However, after the 
initial change to the format, nothing
will visibly change, even 
for paragraphs that are assigned the Body format.
Why? Because when you assign a format to 
a paragraph, you are basically just applying the
properties of the format to the text. Afterwards,
there is no inherent link between the format
and the paragraph. So, this script will also
iterate over all paragraphs in the main flow and
reapply the Body format to all paragraphs
to which it was originally applied.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//For convenience, let's set the 
//format name and font size as variables. That way
//we can easily make the the script work on 
//a different format  and/or size by just making the change here.
var formatName = "Body";
var newFontSize = 24;

//Get the active document
var doc = app.ActiveDoc;

//If we have an active document, let's
//attempt to find the format and make the change.
if(doc.ObjectValid() == true)
{
    //This is a special ExtendScript method for FrameMaker,
    //used to find something that has a "name", like a format.
    var pgfFormat = doc.GetNamedObject(Constants.FO_PgfFmt, formatName);
    
    //If we found it...
    if(pgfFormat.ObjectValid() == true)
    {
        //So, all the characteristics of a paragraph format
        //are known as "properties" of the format. This is similar
        //to how any object has properties. The font size of a format
        //is a simple FontSize property, specified in points.
        
        //The reason we have to multiply by 65536 is complicated.
        //It has to do with the way that certain properties are specified
        //as a "metrics" data type. It's a bit complicated at first.
        //For now, let it suffice to say that if you want to end
        //up with points for a metric-type property, you have to multiply 
        //by 65536. That is, 65536 "metrics" equals 1 pt.
        pgfFormat.FontSize = newFontSize * 65536;
        
        alert("'" + formatName + 
            "' paragraph format successfully set to " +
            newFontSize + " point font. Preparing to update the document...");

        //Now, let's go through the document and reapply the updated
        //paragraph format to all paragraphs that currently use it. Those
        //paragraphs will not update automatically, because there is no
        //direct link between a format and a paragraph to which it is applied.
        //The paragraph will remember the name of the format last applied,
        //but it is not locked to any changes to the format afterwards.
        
        //We'll start by getting the first paragraph in the main flow,
        //then iterate over each paragraph, checking the name of the
        //last format applied, then reapplying the updated format as applicable.
        //To get the first paragraph, notice how we drill down through object
        //properties and "ownership." The doc owns the main flow, the flow owns
        //its first text frame, and a text frame owns its first paragraph.
        var pgf = doc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;
        
        //Run this loop until we run out of valid paragraph objects
        while(pgf.ObjectValid() == true)
        {
            //Let's check the name of the most recently-applied format.
            //(not by the script).
            //If it is the same as the one we just updated, let's reapply.
            if(pgf.Name == formatName)
            {
                //Applying a format programmatically looks much different
                //than doing with with the GUI. What really happens is that
                //you copy all the properties from the format and apply them 
                //to the text in the paragraph. When doing it in a script, you actually
                //have to perform that explicit action. The catalog, etc.
                //in the GUI shield you from all this and just let you 
                //click a button. In ExtendScript, it is still fairly simple, just
                //a call to get the properties then a call to apply them.
                var props = pgfFormat.GetProps();
                pgf.SetProps(props);
            }
        
            //Get the next paragraph in the flow. The current
            //paragraph knows what the next one is.
            pgf = pgf.NextPgfInFlow;
        }
    
        alert("Font change complete.");
    }

    //Otherwise, if we did not find the format, let's report that.
    else
    {
        alert("Could not find the '" + formatName + 
            "' format in the active document.");
    }
}

//Else, if we did not find an active document, let's
//warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");


