﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_UNSTRUCTURED.fm

DESCRIPTION:
    
This is a script that does various navigational
things related to tables and also a little bit
of column resizing for fun.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document.
var doc = app.ActiveDoc;

//If we have an active document, let's
//do the requested task.
if(doc.ObjectValid() == true)
{
    //First, let's find the table in the main flow
    //of the document. There are likely many other
    //tables in the doc, particularly on the reference
    //pages. There are a variety of ways
    //to pull this off, depending on what you are doing and how you
    //have things architected. In this particular document, there
    //is a special table format called "ExtendScriptSample"
    //that is applied to the table in the main flow, which will
    //allow us to positively identify it. So, we'll iterate over
    //all tables in the document (each doc keeps an internal
    //list, and as usual, in no particular order), test the format
    //name, and stop when we find the table of interest.
    var table = doc.FirstTblInDoc;
    
    while(table.ObjectValid() == true)
    {
        //If the table has the tag we are looking for,
        //let's break out of the loop.
        if(table.TblTag == "ExtendScriptSample") break;
        
        //Otherwise, proceed to the next table
        else table = table.NextTblInDoc;
        
    }

    //If we get here with a bogus object, it means we ran 
    //out of tables and never found the one we want.
    //Let's warn.
    if(table.ObjectValid() == false)
    {
        alert("Could not find the table in the main flow. Cannot continue.");
    }

    //Otherwise, let's do some fun stuff.
    else
    {
        //First, let's select the first row and report. Using this table
        //method is fairly straightforward. You just need to specify the bounds
        //of the selection.... that is, the topmost row, the bottommost row,
        //the leftmost column, and the rightmost column. Rows 
        //and columns are indicated by number, where zero is the
        //first column. In this case, we only want to select a single row,
        //so the topmost and bottom most rows are just that row.
        //We want to select the whole row, which means columns 0 through 2
        //(the table has three columns).
        table.MakeTblSelection (0, 0, 0, 2);
        alert("First row in the table selected.");
        
        //Similar thing, let's select the first and
        //second cells of the first and second body rows.
        //Because the first row of the table is the header
        //row, the second table row will be the first body row,
        //number 1.
        table.MakeTblSelection (1, 2, 0, 1);
        alert("First and second cells of the first and second body rows in the table selected.");
        
        //Here's how we can select a column, by querying a 
        //table property to find how many rows there are.
        table.MakeTblSelection (0, table.TblNumRows - 1, 1, 1);
        alert("Second column selected.");
        
        //Here's a trick to select the whole table using a special constant.
        table.MakeTblSelection (Constants.FF_SELECT_WHOLE_TABLE, 0, 0, 0);
        alert("Whole table selected.");
        
        //Let's retrieve the text in the first body cell. For these operations, we will need
        //row and cell IDs. We can get these IDs through property queries.
        
        //Let's start by getting the first body row. We have to start with the
        //first row of the table which is the header row, then drill down
        //one property further to get the next row (the first body row).
        var row = table.FirstRowInTbl.NextRowInTbl;
        
        //Now we can get the first cell in the row.
        var cell = row.FirstCellInRow;
        
        //To retrieve text, we need a text range. A text range
        //indicates a starting and ending point of some text,
        //and includes the paragraph(s) where those points
        //are. All we have now is a cell object, but it provides
        //access to the first paragraph in the cell.
        //As mentioned elsewhere in other scripts, text
        //ranges can be a little tough to understand. It normally
        //takes some time to get a hold of.
        var textRange = new TextRange();
        textRange.beg.obj = textRange.end.obj = cell.FirstPgf;
        textRange.beg.offset = 0;
        textRange.end.offset = Constants.FV_OBJ_END_OFFSET;
        
        //Now that the text range is established, let's get the text
        //from it and report. This requires a text items structure
        //which is even more complicated than text ranges.
        var textItems = doc.GetTextForRange (textRange, Constants.FTI_String);
        
        //Just for aesthetics, let's select the cell too.
        table.MakeTblSelection (1, 1, 0, 0);
        
        
        //And report
        alert("First line of text in first body cell: \n\n" +
            textItems[0].sdata);
            
        //Now, let's replace the text in the first cell. To do this,
        //we need to first delete the existing text, then insert the new
        //text. Text manipulation is a bit complicated as well, so
        //have patience when studying this.
        
        //You have to set up a text range for deletion. We already did
        //that earlier to get the text, so now we can just set the selection
        //in the doc with that range (much like selecting text manually)
        //and then call the Clear() method (much like pressing Delete on 
        //the keyboard).
        doc.TextSelection = textRange;
        doc.Clear(0);
        
        //Now, insert some new text. The paragraph mark should be remaining
        //in the table cell because you can't delete the last paragraph in a cell.
        //The beginning of our text range is already the text location that represents the 
        //beginning of the paragraph, so we can just reuse that for the
        //AddText() method.
        doc.AddText (textRange.beg, "Some new text for the cell!");
        alert("New text inserted in the first body cell.");
        
        //Now let's monkey with some table column widths. First,
        //we'll set the first column to 1 inch.
        
        //The column widths of a table are retrieved and set with the TblColWidths
        //property, which is an array of widths using the concept of "metrics." Metrics
        //are confusing to understand, so the best bet is just to learn the tricks
        //to specify points, inches, or whatever unit you want, and then just reuse them.
        
        //To adjust table column widths, you can start with a completely new widths
        //array, or query the table for the current widths which automatically builds
        //an array of current widths for you. We'll use the latter.
        var colWidths = table.TblColWidths;
        
        //Now, this array should have 3 members, one width for each of the three columns.
        //Let's tinker with the first member to set the width to one inch, then reset the property
        //on the table  to implement the change.
        
        //There are constants defined to help with conversion from various units
        //to the "metric" format. Here's how to specify 1 inch. 
        colWidths[0] = 1 * 4718592;
        //...where the 1 is the number of inches. In other words,
        //there are effectively 4718591 "metrics" in one inch.
        //If you wanted to specify 2.5 inches, you would use:
        //   2.5 * 4718592
        //and so forth. 
        
        //So now, apply the adjusted array to change the table.
        table.TblColWidths = colWidths;
        alert("First column resized to 1 inch.");
        
        //Set the second column to 5.75 centimeters. 
        //There are effectively 1857713 "metrics" in one centimeter.
        colWidths[1] = 5.75 * 1857713;
        table.TblColWidths = colWidths;
        alert("Second column resized to 5.75 centimeters.");
        
        //Now let's make all columns equal. We can do this by
        //finding the total width of the table, then just use a little math.
        var tableWidth = table.TblWidth;
        colWidths[0] = colWidths[1] = colWidths[2] = tableWidth / 3;
        table.TblColWidths = colWidths;
        alert("All columns resized to equivalent widths.");
        
    }
}

//Else, if we did not find an active document, let's
//warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");

