﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (any)

  ...with an anchored frame selected!
  The unstructured sample file will do fine.

DESCRIPTION:
    
    
This is a script that imports a graphic
into an anchored frame and shrinkwraps
the frame around it. It is identical to the
other "import and shrinkwrap" script, 
except this one uses a file browser
to select the file for import. See the instructions
in that other script for more information.

MAKE SURE you have an anchored frame
selected first!


Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document.
//and the currently-selected graphic.
//We will abort if we don't have an active
//doc with a graphic within an
//anchored frame selected, or an
//anchored frame itself selected.

//You don't need an anchored frame to import a graphic.
//We are doing it this way to make the sample more
//straightforward. Without an anchored frame, graphic
//import can be a bit more unpredictable.

//Let's initialize our graphic and frame object variables,
//with the initial assumption that we don't have them. If we find
//the appropriate selections, we'll reset them.
var graphic = null;
var frame = null;

//Look for the active document.
var doc = app.ActiveDoc;

//If we have a document, look for a selected graphic.
if(doc.ObjectValid()) 
{
    //This call will return any selected graphic object,
    //including an anchored frame. 
    graphic = doc.FirstSelectedGraphicInDoc;

    //Now, let's see what we have. We only want to
    //operate if we have a graphic or an anchored frame selected.
    //Here's how we test:. "Inset" means imported graphic.
    //"AFrame" means anchored frame.
    
    //If we have a graphic, great, let's look for the 
    //parent anchored frame, because that's really
    //the item of interest.
    if(graphic.ObjectValid && graphic.constructor.name == "Inset")
   {
       //Here's how to find the parent frame of 
       //a graphic. This might be an anchored frame, but
       //it also might be a text frame on a page, etc.
       frame = graphic.FrameParent;
       
       //Let's make sure it is an anchored frame. If it isn't,
       //reset the variable so the script aborts later. This conditional
       //will be met if the frame object is invalid (ie, there was no frame)
       //or there is a frame but it isn't an anchored frame.
       if(!frame.ObjectValid() || frame.constructor.name != "AFrame")
       {
           frame = null;
       }
   }

    //Otherwise, if the original selection was not a graphic, let's see if it is an 
    //anchored frame.
    else if(graphic.ObjectValid() && graphic.constructor.name == "AFrame")
    {
        //In this case, all we need to do is set the correct variable.
        //An anchored frame is what we really want.
        frame = graphic;
    }
   
     //otherwise, if the selected graphic object is not
     //an anchored frame or a graphic, nothing more needs
     //to be done. The frame variable is currently null, which
     //will abort the script in a bit.

}


//Here is the main import operation. We'll only proceed if
//we found an anchored frame associated with the current
//selection.
if(frame != null)
{
    //We have to use the Import() function to import
    //the graphic. However, this function needs a parameter list
    //as input. Parameter lists can be a bit confusing. The easiest
    //way here is to call a special function to pre-populate a list
    //with default parameters for import.
    var props = new PropVals();
    props = GetImportDefaultParams();
    
    //The default parameters may or may not be OK. 
    //For educational purposes, let's explicitly set
    //the parameter that controls whether the import is 
    //by reference or by copy. We'll set it to by reference.
    
    //The list of parameters is in an array called a "PropVals"
    //structure. Each property has an index. We need to find
    //the index for this particular settings. Here's how. The
    //property for the import method is defined in the global
    //constants list as FS_HowToImport. The GetPropIndex
    //is just a shortcut to sift through the property list and find this one.
    var index = GetPropIndex(props, Constants.FS_HowToImport);
    
    //If we found it, the index will be zero or a positive integer. 
    //Now we can just set the value.
    if(index > -1) props[index].propVal.ival = Constants.FV_DoByRef;
    
    //Constants and property values are tough to figure
    //out. This is just an example to get you started.
    //Expect a learning curve to understand it well.
    //Now that you see some things in action, look them up
    //in the scripting guide (things like "FS_HowToImport").
    //You'll eventually figure out how to adapt this code
    //to do other similar things.
    
    //Let's set a bit more. Look these up
    //to learn more.
    index = GetPropIndex(props, Constants.FS_FitGraphicInSelectedRect);
    if(index > -1) props[index].propVal.ival = false;
    index = GetPropIndex(props, Constants.FS_GraphicDpi);
    if(index > -1) props[index].propVal.ival = 150;

    //Now, import is a simple as a single function call.
    //Note that we have supply the property list and the path
    //of the graphic, but we don't have the path yet. Let's launch a 
    //file browser to get the path. This is a method
    //built into ExtendScript. See the scripting doc for more info.
    var path = ChooseFile("Browse for graphic file", doc.Name, "", Constants.FV_ChooseSelect)    
    
    //If the user did not click cancel, the path variable
    //will be populated with the selected path. Let's import.
    if(path != "")
    {
        //The doc.TextSelection and returnParms parameters
        //are not used in this example, but they are required.
        //If you don't know what they are, you can ignore them for now.
        var returnParms = new PropVals();
        doc.Import(doc.TextSelection, path, props, returnParms);
        
        //...and finally, shrinkwrap. This is really very easy.
        //We'll set the dimensions of the anchored frame
        //to 2pts bigger than the graphic to give a 1 pt 
        //buffer on all sides. Here's a trick... after an import, it's
        //always the imported graphic that is selected, so we
        //can easily get the graphic object to query its dimensions.
        //This is an example of how a knowledge of FM behavior
        //is just as important as coding skills.
        graphic = doc.FirstSelectedGraphicInDoc;
        
        //Height and width values use this concept
        //of "metric" units (not to be confused with the metric
        //system). It's complicated. For the sake of this sample,
        //just know that a metric value of 65535 equals 1 pt.
        //Here, we set the frame height to the same as the
        //graphic height, plus 2 pts. Same wth the width.
        frame.Height = graphic.Height + (2 * 65535);
        frame.Width = graphic.Width + (2 * 65535);
        
        //The frame is the right size. Let's position the graphic
        //at 1pt from the left side and top side of the frame
        //to establish the 1pt buffer on all sides, then we
        //are done.
        graphic.LocX = 65535;
        graphic.LocY = 65535;
    }

}

//Else, if we did not find an active document, land 
//never attempted the socket connection, let's warn.
else
{
    alert("No active document or no graphic or anchored frame selected. " +
        "Cannot continue.");
}

alert("Script complete.");


