﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: (any)

  ...with a graphic or anchored frame selected

DESCRIPTION:
    
This is a script that sizes a graphic to meet
the current dimensions of the parent anchored
frame. It is like a "reverse shrinkwrap,"
a handy little utility in certain situations.

While resizing the graphic, the script
maintains the original aspect ratio. So,
unless the frame has the exact same ratio, 
there will be a gap to the right or below
the graphic following the operation. In a normal
workflow, a shrinkwrap would be applied
afterwards.

You must have an anchored frame or a
graphic selected before running the script.
Experiment with resizing the graphic
and or frame before running the script to
fully appreciate the functionality.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document.
//and the currently-selected graphic.
//We will abort if we don't have an active
//doc with a graphic within an
//anchored frame selected, or an
//anchored frame itself selected.
//In the end, we have to have a graphic
//object and an anchored frame object,
//otherwise there is no point to the operation.

//Let's initialize our graphic and frame object variables,
//assuming at first that we don't have them. If we find
//the appropriate selections, we'll reset them.
var graphic = null;
var frame = null;

//Look for the active document.
var doc = app.ActiveDoc;

//If we have a document, look for a selected graphic.
if(doc.ObjectValid()) 
{
    //This call will return any selected graphic object,
    //including an anchored frame. 
    graphic = doc.FirstSelectedGraphicInDoc;

    //Now, let's see what we have. We only want to
    //operate if we have a graphic or an anchored frame selected.
    //Here's how we test:. "Inset" means imported graphic.
    //"AFrame" means anchored frame.
    
    //If the selection is an anchored frame, let's do this:
    if(graphic.ObjectValid() && graphic.constructor.name == "AFrame")
    {
        //Set the frame variable with 
        //the current graphic object, which is the frame.
        frame = graphic;
        
        //...and see if we have a graphic in the frame.
        graphic = frame.FirstGraphicInFrame;
        
        //If anything is amiss, let's null out the graphic variable
        //to prevent further processing
        if(!graphic.ObjectValid()) graphic = null;
    }
 
    //Otherwise, whatever we have, let's see if it is
    //inside an anchored frame. If so, we will proceed
    //with the operation.
    else if(graphic.ObjectValid())
    {
       //Here's how to find the parent frame of 
       //a graphic. This might be an anchored frame, but
       //it also might be a text frame on a page, etc.
       frame = graphic.FrameParent;
       
       //Let's make sure it is an anchored frame. If it isn't,
       //reset the variable so the script aborts later. This conditional
       //will be met if the frame object is invalid (ie, there was no frame)
       //or there is a frame but it isn't an anchored frame.
       if(!frame.ObjectValid() || frame.constructor.name != "AFrame")
       {
           frame = null;
       }
    }
   
     //otherwise, if the selected graphic object is not
     //an anchored frame or a graphic, nothing more needs
     //to be done. The frame variable is currently null, which
     //will abort the script in a bit.

}


//Here is the main import operation. We'll only proceed if
//we found an anchored frame and graphic associated with the current
//selection.
if(graphic != null && frame != null)
{
    //We really just need to do a little math.
    //Unless the frame and the graphic
    //have the exact same aspect ratio,
    //we can only expand until either the height
    //or the width maxes out at the frame boundaries.
    //So, the first thing we will need to do is determine
    //which bounds will be met first, the height or the width.
    
    //Let's capture all the current height and width
    //values. We'll decrement the frame values
    //by one point on each side because ultimately
    //we'll leave a 2 pt buffer on all sides. In the height
    //and width properties, a value of 65535 equals one point.
    //The use this concept of "metric" values which can be
    //hard to understand at first. To keep it simple here, just know
    //that a value of 65535 equals 1 point.
    var graphicHeight = graphic.Height;
    var graphicWidth = graphic.Width;
    var frameHeight = frame.Height - (2 * 65535);
    var frameWidth = frame.Width - (2 * 65535);
    
    //Now a little bit of math to determine whether
    //the width or the height will max out first. Basically,
    //we'll divide the height by the width for each object
    //to get a real number representation of the aspect ratio.
    //If the frame aspect ratio is "higher" than the graphic ratio,
    //that means the frame has "more heighth" than the graphic
    //and that the width will max out first. And, vice-versa.
    var graphicAspectRatio = graphicHeight / graphicWidth;
    var frameAspectRatio = frameHeight / frameWidth;
    
    //So, here is the case where there is more frame "heighth":
    if(frameAspectRatio > graphicAspectRatio)
    {
        //Fairly simple now. We know the width will be the dimension
        //that maxes out, so we can just change it to the size of the
        //frame width (minus the 2 points we already trimmed off)
        graphic.Width = frame.Width;
        
        //And, since we have the original aspect ratio
        //of the graphic stored, we can use a simple equation
        //set the appropriate height, related to the width resize we 
        //just did.
        
        //If the ratio = height / width, then the height = ratio * width.
        //We have the new width set, so let's just plug it in and
        //get the new height.
        graphic.Height = graphicAspectRatio * graphic.Width;
    }

    //Otherwise, if the frame had more "width," kind of just the
    //opposite operation, but the same logic. This will also
    //handle the case where the aspect ratios are exactly the same.
    else
    {
        graphic.Height = frame.Height;
        
        //If the ratio = height / width, then the width = height / ratio.
        graphic.Width = graphic.Height / graphicAspectRatio;
    }
    
    //The graphic is now the right size. Let's position the graphic
    //at 1pt from the left side and top side of the frame
    //to establish the 1pt buffer on all sides, then we
    //are done.
    graphic.LocX = 65535;
    graphic.LocY = 65535;
}

//Else, if we did not find an active document, land 
//never attempted the socket connection, let's warn.
else
{
    alert("No active document or no graphic or anchored frame selected. " +
        "Cannot continue.");
}

alert("Script complete.");

