﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (any)

DESCRIPTION:
    
This is a script that creates a simple
"MyMenu" menu in the main FM document
and book menus, then adds a simple
"My Command" to it, which pops up
a little message box.  Just for fun,
we'll add a second command too. The point is
to demonstrate basic techniques for
creating menus and commands, then
how to respond to a command once
invoked.

*/


//First, create the menu and commands.
//Menus can "own" other menus and can 
//own commands.

//Let's create the commands first. You can create
//commands and add them to menus at the same time,
//but for functional simplicity, let's do everything
//separately.

//This "DefineCommand" function is built into FM ExtendScript.
//The syntax is: 
//
//     DefineAndAddCommand(cmdID, name, label, shortcut)
//
// ...where:
//    - cmdID - some unique integer that gets send to the Command()
//                       function when the command is invoked. This will make 
//                       more sense later.
// 
//    - name - some unique string that FM maintains as the
//                      the command's internal "name". If you ever
//                      need to access the command object in the future,
//                      you could use this name. More explanation coming up.
// 
//    - label - the string that appears for the command when it is in a menu.
// 
//    - shortcut - the keyboard shortcut to execute the command, if any
//                         (versus selecting it from a menu.) When a shortcut
//                         is defined, it always works, whether or not the command
//                         is ever added to any menu at all. The "\\!" notation means
//                         the Esc key. For example, the shortcut "\\!99" means that
//                         Esc + 9 + 9 will invoke the command.
var cmd1 = DefineCommand(1, "ScriptSampleCommand1", "My Command", "\\!99");
var cmd2 = DefineCommand(2, "ScriptSampleCommand2", "My Other Command", "");

//Now let's create the menu. Very similar sort of operation, except a menu doesn't
//need an ID (no reason to call Command() when a menu is invoked, it just appears)
//and no need to assign a shortcut (not applicable to menus)
var newMenu = DefineMenu("ScriptSampleMenu", "My Menu");

//Let's add the new commands to the new menu. Currently,
//the commands exist, but will not appear anywhere because
//we haven't put them in any menus yet.
newMenu.AddCommandToMenu(cmd1);
newMenu.AddCommandToMenu(cmd2);

//Now we have a menu with two commands, but again,
//the menu will not appear anywhere because we didn't assign it
//anywhere to appear. Menus own other menus, so let's assign
//the new menu to the main document menu in FM and the main
//book menu. That way, it will appear in the main menu
//bar when a book or a document is active.

//To add a menu to a menu, we have to get the parent 
//menu object. We can look up menus by name (similar
//to how we could look up commands by name, if necessary).
//All the default menus in FM have default names, so we can
//look them up if we know the names.
//The name of the main document menu is "!MakerMainMenu"
//and the name of the main book menu is "BookMainMenu."
//How does one know that? Lots of experience :) There are tricks
//to learning all the menu names but there is no documentation
//anywhere about them.
fmMenu = app.GetNamedMenu("!MakerMainMenu");
if(fmMenu.ObjectValid())
{
    fmMenu.AddMenuToMenu(newMenu);
}

fmMenu = app.GetNamedMenu("!BookMainMenu");
if(fmMenu.ObjectValid())
{
    fmMenu.AddMenuToMenu(newMenu);
}

//This is a built-in function that is required to refresh 
//everything and actually make the new menu appear
UpdateMenus();

alert("Script complete! Menu and commands created.");


//Now we have our menus and commands. Great! But, we need
//a mechanism to respond when somebody selects them.
//This is done with the Command() function. This is a built-in function
//that you have to use for responses to menu commands.
//It's operation is simple... any time a command generated by a script,
//this function is called with the command ID (remember the IDs we 
//assigned to commands when we created them?)  That's
//why all IDs must be unique.
//
//All independent scripts that create commands should have 
//their own Command() function. One script cannot respond to
//the invocation of a command created by another script.

function Command(cmd){
    
    //"cmd" is the command ID, assigned when the
    //command was created. The most common way
    //to conditionally respond to an integer-based variable
    //according to the specific integer is with a "switch" statement.
    //This is a standard JavaScript construct and also appears
    //in many other languages. It is not specific to ExtendScript.
    //See a JavaScript tutorial for more information.
    
    //By the way, don't forget to try Esc + 9 + 9 too!
    switch(cmd) 
    {
        //Our first command was given an ID of 1. So, this next code
        //will execute when that command is invoked:
        case 1:
            
            alert("You selected your first script command. Congratulations!");

            break;
            
        //Our second command was given an ID of 2. 
        case 2:
            
            //We'll do just a little more processing for fun. Typically,
            //a script wouldn't put much code here... it would put
            //the code into a separate function and just call the function here.
            //Keeps this area from getting messy.
            alert("You selected your second script command. Congratulations!");
            
            var doc = app.ActiveDoc;
            if(doc.ObjectValid())
            {
                alert("The active document is: \n\n" + doc.Name);
            }
            else
            {
                var book = app.ActiveBook;
                if(book.ObjectValid())
                {
                    alert("The active book is: \n\n" + book.Name);
                }
                else
                {
                    alert("No active book or document.");
                }
            }

            break;
    }
}
