﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (any)

DESCRIPTION:
    
This is a script that creates two "notifications,"
that is, sets up some actions to happen automatically
when certain events happen. In this case, we are
setting up notifications for the events of opening
a document and closing a document. There are
a wide variety of events that a script can respond
to... this is just an example of two to show you 
the general framework. 

The ability to respond automatically to a session
event is a very powerful and versatile feature of 
ExtendScript. You should use it with care, though,
because you could end up creating some kind
of silent, autonomous action and forget about it
later. In other words, if you have gizmos running
on their own, you need to keep track of them. One
technique is to use messaging when autonomous
activity occurs in order to ensure it is desired.

The sample includes provisions to turn off
the notifications that it turns on. Alternatively,
you can restart FrameMaker

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//In general, this is very simple. First, let's turn on 
//notifications for the events we want to respond to.
//The Notification() function does this. It needs only
//two arguments... the event type (specified using the
//global, built-in Constants array) and whether to turn
//the notification on or off (true or false). You can find a full
//list of supported events (and the corresponding Constants value)
//in the ExtendScript documentation.
//
//Note the prefixes "Post" and "Pre" in the names below. For things
//like document opening and closing, you can get notified before or
//after the event. There are reasons why you would want one or the other.
//For example, if you want to do something to a document before it
//closes, you should do it under FA_Note_PreQuitDoc, because at 
//FA_Note_PostQuitDoc, it is already gone.
Notification(Constants.FA_Note_PostOpenDoc , true);
Notification(Constants.FA_Note_PreQuitDoc , true);

alert("Script complete! Notifications are enabled. Open and/or "
    + "close a document to see the effect.");

//That's it! FrameMaker will send a notification  to this script
//whenever a document is opened or closed. How? Using this
//Notify() function. This function is built-in and standard for any notification
//response. If you turn on notifications but do not have this function,
//the notifications will have no effect. 
//
//When FM calls this function, it sends in the following arguments:
// 
// note - The event being notified for (like Constants.FA_Note_PostOpenDoc)
// object - May be populated by some FM object, depending on the type
//                 of event. With a FA_Note_PreQuitDoc notification, 
//                 opening a document, this will be populated with the document object.
//                 For some events, it will not be populated with any object. The documentation
//                 has details.
//  sparam - A string parameter related to the event, if anything. Some events have some
//                     string, some do not. With a FA_Note_PostOpenDoc notification, it is populated
//                     with the document file path. We'll use that in the code below.
//  iparam - An integer parameter related to the event, if anything. Some events have some
//                     integer, some do not. We won't use it in this example.
function Notify(note, object, sparam, iparam)
{
    //Now, recalling that "note" indicates the type of event that occurred,
    //let's check its value and respond appropriately. The standard
    //method for this is a "switch" statement, a common syntax in 
    //the JavaScript language, as well as many other programming languages.
    //It is like a bunch of "if" and "else if" statements, just using a more compact
    //syntax. 
    switch (note) 
    {
        //The "case" is like an "if" statement. Here is the 
        //case to handle the document opening event.
        case Constants.FA_Note_PostOpenDoc:
        
            //We'll present a simple message box with the file name
            //and a question whether the user wants to continue receiving 
            //the message box. If the user clicks No, we'll capture that
            //and turn off the notification. So, this sample includes turning
            //off a notification as well as turning it on.
            var returnVal = confirm("You just opened: \n\n" +
                sparam + "\n\nDo you want to keep receiving this notification?");
                
            //When the user clicks No, the confirm() function returns zero. So, if 
            //that happened:
            if(returnVal == 0)
            {
                //Turning off a notification is as simple as turning it on.
                Notification(Constants.FA_Note_PostOpenDoc , false);
            }
                
            //A case construct is usually terminated with a break statement,
            //to prevent any other cases from being processed. This is
            //a characteristic of the switch construct and a more advanced 
            //programming concept. For the sake of this sample, just know that
            //it needs to be here.
            break;
  
        //Here is the case to handle the document close event.
        case Constants.FA_Note_PreQuitDoc:
        
            //The only difference here is that we'll use the document
            //object to get the file path. So let's get that object.
            var doc = app.ActiveDoc;
            
            //Otherwise, everything is basically the same as the open doc routine above
            var returnVal = confirm("The following document is about to close: \n\n" +
                doc.Name + "\n\nDo you want to keep receiving this notification?");
                
            if(returnVal == 0)
            {
                Notification(Constants.FA_Note_PreQuitDoc , false);
            }
                
            break;
     }
}

