﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleBook.book

DESCRIPTION:
    
This is a script that does a few things related
to book navigation and file opening, including:

 - Walking through a book one component at time,
    selecting and reporting on each component

 - Opening each file in a book and deleting one 
   invalid component
 
 - Updating the book

Books and book components can be a little tricky to manage
until you understand the underlying architecture.
Basically, a book contains a set of
"components", which may include
files, folders, and/or groups. A file component is
just a pointer to a file... it isn't really the file
itself. So, when dealing with file components
in a book, you have to understand that a component
object is just that... a component, with properties
and methods related to components. If you want
to manipulate the properties of the respective document,
you have to open the file and get the actual file
object.

*/

//Let's get the active book.
var book = app.ActiveBook;

//If no book was active, let's look for any book and hope
//it is the sample.
if(!book.ObjectValid()) 
{
    book = app.FirstOpenBook;
}

//If we have a book, let's do the rest of the tasks.
if(book.ObjectValid())
{
    //Let's make sure the book is the active file.
    //It will make sure it is in front and we can see
    //the things that the script does.
    //If a document is currently active, this call will
    //remove the focus from that document.
    app.ActiveBook = book;

    //The remainder of the tasks involve walking though the book.
    //Walking through a book is generally simple. The book
    //object has a "first component" property and each component
    //has a property that points you to the next component.

    //Let's start with a simple walk through the active book
    //and report some details on each component.
    var comp = book.FirstComponentInBook;
    
    //We'll use a counter to help with messaging
    var counter = 0;
    
    while(comp.ObjectValid())
    {
        //Increment the counter.
        counter++;
        
        //Let's start off this message for our prompt.
        //All prompts will start off the same way,
        var msg = "Component #" + counter + "\n\n";
        
        //Select the current component so we can see where we are.
        comp.ComponentIsSelected = true;
        
        //Let's find out what kind of component it is,
        //then add to our message appropriately.
        var compType = comp.ComponentType;
        
        if(compType == Constants.FV_BK_FILE)
        {
            msg += "Type: File\n\nPoints to:\n\n  "
                + comp.Name;
        }
        else if(compType == Constants.FV_BK_GROUP)
        {
            msg += "Type: Group\nName: " 
                + comp.BookComponentTitle;
        }
        else
        {
            msg += "Type: Folder\nName: " 
                + comp.BookComponentTitle;
        }
    
        alert(msg);
        
        //Let's deselect the current component 
        //to get ready for the next one.
        comp.ComponentIsSelected = false;
        
        //...and get the next one. The "DFS order"
        //bit means "Depth-first search", a fancy term for
        //a method of exploring down every branch in a tree
        //in a logical fashion. From Wikipedia:
        
        //"Depth-first search (DFS) is an algorithm for traversing 
        //or searching tree or graph data structures. 
        //One starts at the root (selecting some arbitrary node 
        //as the root in the case of a graph) and explores as far 
        //as possible along each branch before backtracking."
        // (http://en.wikipedia.org/wiki/Depth-first_search)
        
        //Note that we are asking the component for the next
        //component, not the book. The book doesn't know where
        //we are right now, but each component knows
        //who is the next component relative to itself.
        comp = comp.NextBookComponentInDFSOrder;
    }


    //Let's walk through the book again and attempt to open
    //each component file.
    var comp = book.FirstComponentInBook;
    
    while(comp.ObjectValid())
    {
        //Attempt to open the file, if the current component
        //is a file component. The Open() function is defined
        //later in the script... see there for lots more info.
        if(comp.ComponentType == Constants.FV_BK_FILE)
        {
            var doc = OpenFile(comp.Name);
        
            //Let's report what happened!
            if(doc.ObjectValid())
            {
                alert("Successfully opened: \n\n" + comp.Name);
            }
            else
            { 
                alert("Could not open: \n\n" 
                    + comp.Name
                    + "\n\nMaybe the file doesn't exist?\n\n"
                    + "We are going to delete this component now "
                    + "because we want to do a book update, but " 
                    + "it will fail with this bogus component.");
                    
                //We're going to delete the component. So that
                //we don't lose our place in the step through the
                //book, let's put the current component object in a 
                //temporary buffer, then reset the main object
                //variable ('comp') to the previous component.
                //This way, our loop will continue to process normally. 
                //Otherwise, we will have effectively deleted the step
                //we are on and basically fallen into nowhere.
                var compToDelete = comp;
                comp = comp.PrevBookComponentInDFSOrder;
                
                //Just in case it was already the first component, ie,
                //no previous component
                if(!comp.ObjectValid()) comp = comp.FirstComponentInBook;
                
                //Delete.
                compToDelete.Delete();
            }
        }
    
        comp = comp.NextBookComponentInDFSOrder;
    }

    //Let's do a book update using default parameters.
    //You can control a wide variety of update parameters
    //just like you can in the interface, but for simplicity, we'll
    //use the defaults (mostly), which basically say to update everything.
    //Ignore the "returnp" argument for now. It is a required array
    //that captures information that occurred during the update.
    //It's complicated and we won't use it here. The method requires
    //it but you are not required to do anything with it afterwards.
    
    //We are going to change one default, which is related to the
    //book error report. By default, the report does not appear for
    //a book update initiated by a script. There is a cross-reference
    //error intentionally placed in the sample files, so let's enable
    //the report. This part is a little complicated so we are going to
    //skip a detailed explanation for now. If it makes no sense to you,
    //just browse over it for the time being.
    var props = GetUpdateBookDefaultParams();
    var index = GetPropIndex(props, Constants.FS_ShowBookErrorLog);
    props[index].propVal.ival = true;
    var returnp = new PropVals();

    alert("Preparing to update the book!");
    
    book.UpdateBook(props, returnp);
    
    alert("Book update complete!");

}
else
    alert("No book is open or active. Cannot run the script.");
    
    
 
 //A convenience function for opening a file based on a file
 //path. It generally ignores any errors such as missing
 //images, improper FM version, etc.
function OpenFile(path)
{
    var index;

    //When opening a file, a script uses a long list
    //of parameters to control its behavior. This is necessary
    //because of the wide variety of scenarios that could
    //be encountered and the differing ways that they could be handled.
    //If unspecified, FM uses a default setting for each parameter, which
    //may or may not be what you want in any given case. In this script, we
    //will explicitly set the parameters we know are important to us.
    
    //Later, when we call the Open() method to open the file, we will
    //need to provide the list of parameters we want. This is done
    //with a "PropVals" array, a typical method for specifying long parameter
    //lists in a script. We can build one ourselves from scratch, or we can call
    //this next convenience function (built-in) to populate an array with the
    //entire list of available parameters and their respective default values.
    
    //PropVals arrays are hard to understand for the beginner. If you are
    //new, keep this on a high level and just use this example as a template
    //if you need similar functionality. Eventually, it will start to make more sense.
    props = GetOpenDefaultParams();

    //Now, we have a long list of parameters in a PropVals array (props).
    //If we know the index of any particular parameter in the array, we can
    //get in there and set it to some desired value. ExtendScript provides 
    //a convenience function for this, GetPropIndex(), which looks up an index
    //based on the property "name". You could write your own function to do
    //this but  there is no need since it is provided.
    
    //Properties are identified by an item in the built-in Constants list, which
    //is actually just a list of numbers. The use of semantically-named 
    //numerical constants shields us from the need to be looking at and
    //remembering a zillion hexadecimal numbers. Again.... some complexities
    //here that you don't really need to understand to make this work at
    //a basic level. Just follow this model for whatever file opening
    //scenario you want. The rest will come later.
    
    //NOTE: There are more parameters available than are shown here!
    //This is just a selection of common parameters you might want
    //to set. Review the documentation for information on all parameters
    //related to opening files.
    
    //Let's turn off any notifications for general failures.
    index = GetPropIndex(props, Constants.FS_AlertUserAboutFailure);
    if(index > -1) 
        props[index].propVal.ival = false;
    
    //If the document or book is in use by another user, let's 
    //let's reset the lock and open it anyway.
    index = GetPropIndex(props, Constants.FS_FileIsInUse);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_ResetLockAndContinue;

    index = GetPropIndex(props, Constants.FS_BookIsInUse);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_ResetLockAndContinue;

    //Continue anyway if a lock cant be reset
    index = GetPropIndex(props, Constants.FS_LockCantBeReset);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;
        
    //Let's allow the opening of MIF and XML files
    index = GetPropIndex(props, Constants.FS_DisallowMIF);
    if(index > -1) 
        props[index].propVal.ival = false;
        
    index = GetPropIndex(props, Constants.FS_DisallowXml);
    if(index > -1) 
        props[index].propVal.ival = false;
        
    //Let's allow the opening of documents from older FM versions        
    index = GetPropIndex(props, Constants.FS_FileIsOldVersion);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;
        
    //Ignore the "changed metric" error        
    index = GetPropIndex(props, Constants.FS_FontChangedMetric);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;

    //Ignore the "missing fonts" errors
    index = GetPropIndex(props, Constants.FS_FontNotFoundInCatalog);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;
        
    index = GetPropIndex(props, Constants.FS_FontNotFoundInDoc);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;
 
    //Ignore the "missing language" error
    index = GetPropIndex(props, Constants.FS_LanguageNotAvailable);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;
        
    //Set the flag to automatically recognize the file type
    //(.fm, .xml, etc.) or at least try
    index = GetPropIndex(props, Constants.FS_OpenAsType);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_AUTORECOGNIZE;
        
    //Let's not update any text insets or xrefs upon opening       
    index = GetPropIndex(props, Constants.FS_UpdateTextReferences);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoNo;
         
    index = GetPropIndex(props, Constants.FS_UpdateXRefs);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoNo;
    
    //Let's not use any auto-recovery files, if they exist
    index = GetPropIndex(props, Constants.FS_UseRecoverFile);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoNo;
        
    index = GetPropIndex(props, Constants.FS_UseAutoSaveFile);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoNo;
    
    //If the file is in a non-writeable location, let's open it anyway.
    index = GetPropIndex(props, Constants.FS_OpenFileNotWritable);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;

    //Ignore any missing referenced files
    index = GetPropIndex(props, Constants.FS_RefFileNotFound);
    if(index > -1) 
        props[index].propVal.ival = Constants.FV_DoOK;
        
  
    //Finally, we are ready open the document! There is just one more
    //thing we need for the Open() method... another PropVals structure
    //to capture details on the events that occur during the open attempt.
    //If something goes wrong during the attempt, the script will populate
    //this array with some information about it. This is an advanced topic
    //and we will not deal with the granular results of the operation
    //in this script. However, the Open() method requires the array,
    //so we will give it one but just ignore it afterwards. It will just go away.
    returnp = new PropVals();
    var file = Open(path, props, returnp);
 
    //That's it! The Open() method returns a file object if the
    //file opened, or if not, an invalid object variable. Let's send it
    //back.
    return file;
}    

alert("Script complete!");