﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_Structured.fm

DESCRIPTION:
    
This is a script that  walks through the structure
tree in the main flow, stops and selects each
element, and sets an attribute on all <Body>
elements to trigger an EDD rule that applies
a blue format. The key activities demonstrated are:

- How to walk through a tree
- How to select an element
- How to set an attribute




*/

//Get the active document.
//and the highest-level element
//of the main flow
var doc = app.ActiveDoc;
if(doc.ObjectValid())
  var elem = doc.MainFlowInDoc.HighestLevelElement;
else var elem = null;

//If we have a valid highest-level element, that means
//we have a structured flow and can continue.
if(elem != null && elem.ObjectValid()) 
{
    //We're going to walk through the entire element tree
    //and do a couple things along the way.
    //Let's control this with a "while" loop.
    while(elem.ObjectValid())
    {
        //Let's select the current element. This will select it in
        //both the document window and the Structure View,
        //just like a manual selection.
        //We'll use a custom function (defined later in the script)
        //for this. It uses the complicated concept
        //of element ranges. Don't spend too much time trying
        //to figure it out at this point if you are a beginner.
        SelectElement(doc, elem);
        
        //Now let's get the element tag to prepare for a prompt.
        //The tag name is actually a property of the element definition,
        //which is a property of the element. In other words, the element
        //doesn't know the tag name. The element knows which element
        //definition from the EDD is applied to it, and the definition knows
        //its tag. This is all to say that element definitions are separate
        //objects apart from elements and we have to drill down to get the
        //tag name.
        var tag = elem.ElementDef.Name;
        
        //Now one more fancy thing just for fun. Let's count the number of 
        //children for the prompt. We'll just get the first child, then
        //step down the children and increment a counter with each one,
        //until we run out.
        var numChildren = 0;
        var childElem = elem.FirstChildElement;
        while(childElem.ObjectValid())
        {
            //increment the counter            
            numChildren++;
            
            //Get the next child element. Note that a child
            //element is our current reference point, so we
            //actually want a sibling to the current element.
            //We are literally just walking down the tree.
            childElem = childElem.NextSiblingElement;
         }
     
         //Now a little prompt. We'll use a Yes/No prompt
         //so the script can be canceled.
         var message = "Element tag: <"
             + tag
             + ">\nNumber of children: "
             + numChildren
             + "\n\nWould you like to continue the walk?";
         
         //The confirm() function produces a Yes/No
         //prompt and returns zero if the user clicks No.
         if(!confirm(message))
         {
             alert("Script canceled.");
             
             //Null this out to serve as a flag later
             elem = null;
             
             //The break statement will get us out 
             //of the immediate parent loop, which 
             //in this case is the while loop (and the 
             //only loop). That will effectively end the script.
             //The use of break statements
             //is not always considered good practice 
             //because abrupt exits from routines can be
             //hard to keep track of. We'll use it here for
             //simplicity and because it is valid syntax.
             break;
         }
     
         //For <Body> elements, let's set the "Props" attribute
         //to "Blue". This is just to show an example of setting
         //an attribute. We'll use a custom function for this,
         //defined later. The EDD is set up to apply a different
         //paragraph format once this attribute is set, to help
         //visually see the action.
         if(tag == "Body")
         {
             SetAttribute(elem, "Props", -1, "Blue");
             
             alert("Detected a <Body> element, set the "
                 + "'Props' attribute to 'Blue'.");
         }
             
         //Let's get the next element in the tree and 
         //continue on. To learn about "DFS order",
         //look up "depth-first searching" on
         //wikipedia.
         elem = elem.NextElementDFS;
     
     } //end of the while loop    
    
     //If we get here without a null element object,
     //the script completed without cancellation.
     if(elem != null) alert("Script complete!");

}


//Else, if we did not get a valid highest-level element
else
{
    alert("No active document or no highest-level element could be found. " +
        "Cannot continue.");
}


//Here is a function that will select a single element in its
//entirety. This involves text ranges and element locations,
//some very tricky stuff even for the seasoned FM developer.
//The way it usually work is that you get (or "borrow") something
//that works, then just keep using it.
//Comments here are kept to a minimum because 
//this is advanced scripting. Feel free to borrow :)
function SelectElement(doc, elem)
{
    //Abort if the element object is not valid
    if(elem == null || !elem.ObjectValid()) return;
    
    var er = new ElementRange();
    
    //Two different ways here, on for "tagged" elements
    //and one for text nodes. We can determine whether
    //we have a text node by the presence of an element tag.
    
    //if it's a text node, we need to simply retrieve the element
    //text range and convert it. This is the same for selecting
    //the element or contents, because the element is the contents
    if(!elem.ElementDef.ObjectValid() || 
        elem.ElementDef.Name == null ||
        elem.ElementDef.Name == "")
    {
        er.beg = doc.TextLocToElementLoc(elem.TextRange.beg);
        er.end = doc.TextLocToElementLoc(elem.TextRange.end);
    }
    
    //Otherwise, some fancy footwork to set up the element range
    else
    {
        er.beg.parent = er.end.parent = elem.ParentElement; 
        er.beg.child = elem;
        er.end.child = elem.NextSiblingElement;
        er.beg.offset = er.end.offset = 0;
    }

    //Set the document element selection to what we got.
    doc.ElementSelection = er;
}


//Sets an attribute value by name or by index.
//Send an index or an attribute name, not both.
//The name is case sensitive.
//Returns true if successful, false otherwise
//It overwrites whatever is already there.
function SetAttribute(elem, attrName, attrIndex, newValue)
{
    if(elem == null || !elem.ObjectValid()) return false;
    
    var returnVal = false;
    
    var attrs = elem.Attributes;

    for(var i = 0 ; i < attrs.length; i++) 
    {
        if (attrs[i].name == attrName ||
            i == attrIndex) 
        {
            attrs[i].values = new Strings();
            attrs[i].values.push(newValue);
            
            elem.Attributes = attrs;
            i = attrs.length;
            returnVal = true;
        }
    }

    return returnVal;
}


