﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (any)

DESCRIPTION:

This script creates very basic palette (modeless dialog box)
with two buttons, using a coordinate-based layout.
The point is to demonstrate the very basics of creating
a user interface, which can get enormously complicated
depending on what you want to build.

This palette is resizable but the control layout is rigid and will
not respond to resizing.
There is an alternative method of building a palette 
that uses a less rigid layout approach where components
can reposition and resize automatically. This is addressed
in another sample.

See the JavaScript Tools Guide under the Help
menu of the ExtendScript Toolkit window for more information
on palettes and other user interface controls. This information
is not in the FrameMaker-specific scripting guide.


*/

LaunchPalette();

function LaunchPalette()
{
    //Use the Window class to create the main palette. The "palette" argument
    //makes it a modeless dialog, The "My Graphic Utilities" argument defines
    //the title for the dialog. [100, 100, 340, 260] are coordinates that define
    //the initial position and size of the dialog, where the numbers (in pixels) are:
    //
    // [upperLeftX, upper leftY, lowerRightX, lowerRightY]
    //
    //...so the resulting affect is that the top left of the palette will appear at
    //at coordinates 100, 100, where 0,0 is the top left of the screen, and the
    //bottom right corner is at 340, 160. With that, the palette will be 240 pixels
    //wide and 160 pixels tall.
    //
    //When the controls are added to the palette, they all have coordinates that
    //operate in a similar manner, except that the 0,0 reference point changes
    //to the upper left corner of the palette, etc. These types of windows
    //also have a comprehensive feature set that allows automatic resizing
    //and alignment, but for simplicity, everything in this sample is a fixed
    //position and size.
    palette = new Window("palette", "My Palette", [100, 100, 260, 140], {resizeable:true});  
    
    //Define the two buttons for the palette, using a coordinates to determine
    //their sizes and locations relative to the upper left corner of the palette.
    palette.helloButton = 
      palette.add("button", [10, 10, 70, 30], "Say Hello");

    palette.closeButton = 
      palette.add("button", [85, 10, 145, 30], "Close");

    //Now establish the "callbacks", that is, functions that execute when
    //controls are operated. For example, this first one defines what to do
    //when somebody clicks the "Say Hello". There are a wide
    //variety of user actions that you can respond to... in this case, we only
    //care about a click. Because we have not specified any code for any of
    //the other types of actions (callbacks), nothing happens when those
    //actions are performed.
    palette.helloButton.onClick = function() 
    {
        //Just for fun, let's query some session 
        //properties for the message :)
        alert("Hi there! You are using FrameMaker version "
            + app.VersionMajor + "." + app.VersionMinor + "." + app.VersionRevision
            + ".\n\nYour username/login is \"" + app.UserLogin + "\"."
            + "\n\nThe product is installed at the following location:\n\n  "
            + app.HomeDir
            + "\n\nHave a nice day!");
	};

    //Callback for clicking the Close button, just want to close the palette.
    palette.closeButton.onClick = function() 
    {
        palette.close();
        palette = null;
	};

    //This gets called by close() above and when
    //you click X in the upper right corner.
    palette.onClose = function() 
    {
        palette = null;
	};

    //Finally, after all is configured and defined, make the palette appear 
    //on screen.
    palette.show();
    
 }




//****************************************
// FUNCTIONS

//This function increase or decreases the height and/or width of
//a graphic object,. Arguments are:

// - graphic - A graphic or anchored frame object. You can also send
//                     null to assume the currently-selected object, if any.
// - height_inc - The amount of centimeters to increase or decrease
//                          the height. Send zero to leave the height as-is
// - width_inc - The amount of centimeters to increase or decrease
//                          the width. Send zero to leave the width as-is
// - expand - Flag that indicates whether you want to expand or shrink
//                     the object by height_inc and width_inc. Send true to do
//                     an expansion, false to do a shrink
function SizeGraphic(graphic, height_inc, width_inc, expand)
{
    //If null was sent for the graphic, attempt to derive it from the current
    //selection in the active document. If we still can't get a graphic object,
    //exit the function.
    if(graphic == null)
    {
        var doc = app.ActiveDoc;
        if(doc.ObjectValid()) graphic = doc.FirstSelectedGraphicInDoc;
        if(!graphic.ObjectValid()) return;
    }

    //if we are expanding the object, add the increment amounts to
    //the height and width, respectively. To convert centimeters
    //to the point value that these properties use, we have to multiply
    //by 1857710 (that is, 1857710 points = 1 cm).
    if(expand)
    {
        graphic.Width = graphic.Width + (width_inc * 1857710);
        graphic.Height = graphic.Height + (height_inc * 1857710);
    }
    
    //if shrinking the object, similar concept, except subtract the values
   else
    {
        graphic.Width = graphic.Width - (width_inc * 1857710);
        graphic.Height = graphic.Height - (height_inc * 1857710);
    }
}

//This function shrinkwraps an anchored frame around a graphic
//leaving the amount of space specified by 'buffer' (in cm) on all
//sides. The buffer can be zero. If null is sent for the anchored
//frame and/or the graphic, the current selection is used, if
//any
function ShrinkwrapGraphic(aframe, graphic, buffer)
{
    //if the aframe and/or graphic were sent here as null, 
    //use these custom functions to attempt to derive from
    //the current selection. In the end, if we don't have both,
    //we'll exit the function
    if(aframe == null) aframe = GetSelectedAFrame (null);
    if(graphic == null) graphic = GetSelectedGraphic(null);
    
    if(graphic == null || aframe == null) return;
    
    //First, let's size the anchored frame. We'll set the height and width
    //to the same as the graphic, adding additional space for the specified
    //buffer. We multiply the buffer times 1857710 to convert cm to
    //points, which is the unit for these properties. Also, we multiply
    //by 2 because we need the buffer on both sides (top and bottom, 
    //left and right)
    aframe.Width = graphic.Width + (2 * buffer * 1857710);
    aframe.Height = graphic.Height + (2 * buffer * 1857710);
    
    //Now we need to position the graphic, because although the
    //anchored frame is the right size, the graphic could be floating
    //around anywhere in there. The position of a graphic is denoted
    //by the LocX and LocY properties, which are simple x/y coordinates
    //that represent the respective offsets from the bottom left corner of the
    //frame. So, it's simple... we want the coordinates to be 0,0 to align it exactly
    //in the corner, plus any offset specified by the buffer to create the offset effect
    //on all sides.
    graphic.LocX = buffer * 1857710;
    graphic.LocY = buffer * 1857710;
}

//This function sizes a graphic to fit exactly within its anchored frame,
//without changing any of the dimensions of the frame.
//It preserves the aspect ratio of the graphic, so if the graphic
//and frame have different aspect ratios, there will be empty
//space on the top or right after the operation, depending on 
//which boundary was reached first.
function SizeToFitGraphic(aframe, graphic, buffer)
{
    //If null was sent for the graphic and/or aframe, 
    //attempt to derive them from the current
    //selection in the active document. If we still can't get the objects,
    //exit the function.
    if(aframe == null) aframe = GetSelectedAFrame (null);
    if(graphic == null) graphic = GetSelectedGraphic(null);
    
    if(graphic == null || aframe == null) return;
    
    //Let's get the aspect ratios of the graphic and the frame. These
    //will be used for calculations and will not change throughout the 
    //operations
    var aframeAspectRatio = aframe.Height / aframe.Width;
    var graphicAspectRatio = graphic.Height / graphic.Width;
    
    //If the height/width aspect ratio of the frame is less than the 
    //graphic, the maximum height of the graphic will be reached
    //before the maximum width. That is, if the graphic were
    //stretched manually, the height would reach the anchored frame
    //height before the width. So, we will end up with some empty space 
    //on the right after it is over.
    if(aframeAspectRatio < graphicAspectRatio)
    {
        //We know the height is going to max out first, so we can go ahead
        //and set it to the height of the frame, minus any buffer that
        //was specified.
        graphic.Height = aframe.Height - (2 * buffer * 1857710);
        
        //Now, just a little math. To preserve the aspect ratio based 
        //on the new height, we can divide the new height
        //by the aspect ratio to get the required new width.
        graphic.Width = graphic.Height / graphicAspectRatio;
    }

    //Same concept if the graphic aspect ratio is less than the
    //frame aspect ratio, except we will do the opposite.... set 
    //the width based on the frame width and then a little
    //math to derive the proper height to ensure the same
    //aspect ratio.
    else 
    {
        graphic.Width = aframe.Width - (2 * buffer * 1857710);
        graphic.Height = graphicAspectRatio  * graphic.Width;
    }
    
    //Now just like shrinkwrapping, we have the graphic at the size we 
    //want, but it could be floating around anywhere. Let's position
    //it properly at the 0,0 coordinate, plus any offset required by 
    //the specified buffer
    graphic.LocX = buffer * 1857710;
    graphic.LocY = buffer * 1857710;
}

//This function retrieves the currently selected
//graphic object. If an anchored frame is selected,
//it returns the first graphic it finds in the frame,
//if any. If nothing is ever found, it returns null.
function GetSelectedGraphic(doc)
{
    //If null was sent for the document, assume the
    //active document. If there is no active document,
    //exit the function.
    if(doc == null) doc = app.ActiveDoc;
    if(doc == null) return null;
   
    //Get the first selected graphic in the document    
    graphic = doc.FirstSelectedGraphicInDoc;
   
    //If we don't have anything selected, it's 
    //automatically a bust.
    if(!graphic.ObjectValid()) graphic = null;
    
    //If we had an anchored frame selected, let's
    //look deeper to find the first graphic in the frame.
    else if(graphic.type == Constants.FO_AFrame)
    {
        graphic = graphic.FirstGraphicInFrame;
        if(!graphic.ObjectValid()) graphic = null;
    }
    
    //Return whatever we have.
    return graphic;
}

//This function retrieves the currently-selected
//anchored frame, or the anchored frame
//that contains the currently-selected graphic.
//If no frame can be found, it returns null.
function GetSelectedAFrame(doc)
{
    //If null was sent for the document, assume the
    //active document. If there is no active document,
    //exit the function.
    if(doc == null) doc = app.ActiveDoc;
    if(doc == null) return null;
   
    //Get the first selected graphic in the document.
    graphic = doc.FirstSelectedGraphicInDoc;
   
    //If nothing is selected, this is automatically a bust.    
    if(!graphic.ObjectValid()) graphic = null;
   
    //If we do have something selected but it is not an 
    //anchored frame, let's walk up to the parent frame
    //and see if it is an anchored frame. If
    else if(graphic.type != Constants.FO_AFrame)
    {
        graphic = graphic.FrameParent;
        
        //If we didn't find any parent or the parent is not
        //an anchored frame, reset the return variable to null
        if(!graphic.ObjectValid()) 
            graphic = null;
        else if(graphic.type != Constants.FO_AFrame)
            graphic = null;
    }
    
    //Return whatever we've got.
    return graphic;
}




