﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_UNSTRUCTURED.fm

DESCRIPTION:
    
This is a script that will retrieve text from a file at www.weststreetconsulting.com
and replace the first paragraph in the sample document with that text..
The text in that file is simply:

TEXT FROM THE INTERNET!

...without any HTML markup or other content. The script is
intended to demonstrate how to retrieve data over an HTTP socket.
In this case, we are accessing an internet location, but it could be
any accessible network location, such as a place on an intranet.

As written, you must have internet connectivity for
this script to work! More specifically, you must be able to access this URL:

http://www.weststreetconsulting.com/resources/es_sample.txt

There are a number of modest complexities involved
with the concepts involved here. To understand everything
that is going on, you need to have some baseline understanding
of the HTTP protocol and JavaScript string parsing. This sample
does not attempt to serve as a tutorial for either. If you want to know
more, you'll need to study up on your own.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document.
var doc = app.ActiveDoc;

//If we have an active document, let's
//do the requested task.
if(doc.ObjectValid() == true)
{
    //Alert about what we are getting ready to do.
    alert("Sending a request over the internet to the following URL:\n\n" + 
        "www.weststreetconsulting.com/resources/es_sample.txt");
        
    //Attempt to open our socket connection. This
    //is a special type of ExtendScript object used
    //for HTTP sockets. It is described in the ExtendScript
    //JavaScript Tools Guide (available from the Help menu
    //of the ESTK script editor).
    var conn = new Socket;
    if (conn.open ("www.weststreetconsulting.com:80", "UTF-8"))
    {
        //If we get here, we were able to negotiate with the website
        //and open a socket connection. Let's
        //form our "GET" request to ask for a file named 'es_sample.txt'
        //from the site, under the 'resources' folder. A GET request is what
        //a browser sends when you enter a URL into the navigation bar.
        //What you see here is really just what a browser does, except behind
        //the scenes. In fact, you can access this content yourself with a browser
        //by entering the following URL:
        //
        //  http://www.weststreetconsulting.com/resources/es_sample.txt
        // 
        //If you don't know a lot about HTTP, the syntax here won't make 
        //any sense. You'll need to study up on HTTP if you want to know more.
        var getRequest = "GET /resources/es_sample.txt HTTP/1.1\r\n" +
            "Host: www.weststreetconsulting.com\r\n\r\n";
        
        //Set this setting so that line break characters behave correctly. This is 
        //a little complicated... see the ExtendScript documentation if you want
        //to know more.
        conn.encoding = "BINARY";
        
        //...and submit our request to the web server over our socket connection. 
        //If everything is right, the server will
        //send us back the full contents of the es_sample.txt file,
        //preceeded by some HTTP response header information.
        conn.write(getRequest);
        
        //Now that we've written the request to the socket,
        //let's call the read() method to get the web server's
        //response. There is some complexity here because
        //the full response includes a header, but we
        //don't want that in the final text we insert in the doc.
        //In other words, we are most interested in the content
        //of the file we requested. However, the header contains
        //an important field that we need to know - Content-Length,
        //which indicates the number of bytes in the content of
        //that file. We want to know that so we don't sit 
        //around waiting for more characters with the read() method.
        //
        //So, this first part is to read in just the header. The header
        //is separated from the actual content by two 
        //carriage return / line feed combinations (CRLF or \r\n).
        //So, we'll read in characters from the server until we 
        //get there, then we have the header. Then, we'll parse
        //out the content length.
        //
        //We run a little counter to make sure we don't enter an
        //endless loop if something goes amiss with the network.
        var endlessLoopCounter = 0;
        var responseHeader = "";
        
        //This while loop will end when the end of the string is "\r\n\r\n",
        //signaling the end of the header is reached.
        while(responseHeader.substring(responseHeader.length - 4, 
            responseHeader.length) != "\r\n\r\n")
        {
            //Read in a single character and add it to our running string
            responseHeader += conn.read(1);
            
            //Check for an endless loop. We'll allow 1,000 characters to
            //go by before we decide there is a critical problem.
            if(endlessLoopCounter++ > 1000)
            {
                alert("Something unusual happened while attempting to " +
                    "retrieve the HTTP response header. This operation " +
                    "will not behave as expected.");
                
                //break out of the while loop.
                responseHeader = "";
                break;
            }
        }
    
        //Now, a little trickery to read the numerical value
        //in the Content-Length field in the header, so we can
        //efficiently read in the rest of the content. We'll report
        //the full header later, so you can see what a header 
        //looks like. 
        //We'll use some fancy JavaScript footwork here
        //to do this... study up on JavaScript to understand it better.
        //It's just a bit of string parsing. If you were to do this with any 
        //frequency, the right way to do this would be to build
        //a separate function to parse the header and return
        //the desired values.
        var contentLength = 0;        
        var index1 = responseHeader.toLowerCase().indexOf("content-length:");
        var index2 = responseHeader.toLowerCase().indexOf("\r\n", index1);
        if(index2 < 0) responseHeader.toLowerCase().indexOf("\n", index1);
        if(index1 > -1)
        {
            contentLength = parseInt(responseHeader.substring(index1 + 15, index2));
        }
        else 
        {
            alert("A problem occurred while attempting to determine the " +
                "content length of the HTTP response. This operation " +
                "will not behave as expected.");
        }
    
        //Now, let's read the rest of the content, the actual content
        //we wanted to retrieve. We know the length as parsed out
        //of the header, so let's run this loop to get that many
        //characters, which will get us right to the end.
        var content = "";        
        for(var i = 0; i < contentLength; i++)
        {
            content += conn.read(1);
        }
    
        //A little messaging if something went wrong
        if(content == "")
        {
            alert("A problem occurred while attempting to read " +
                "the web content. This operation " +
                "will not behave as expected.");
                
            content = "Error!";
        }
        
    
        //We can close our socket connection now.
        //It would time out eventually but this is good practice.
        conn.close();
        
        //Let's report the entire response from the web server,
        //including the HTTP response header
        alert("Here is the full response from the web server, including the " +
            "HTTP response header. There should " + 
            "be an HTTP response code of '200 OK', otherwise " +
            "something went wrong.\n\n" + responseHeader + content);
            
        //Now let's also report the content from the file we requested.
        //If everything goes right, we should have been able to parse out
        // TEXT FROM THE INTERNET!
        alert("Here is the actual content we wanted to retrieve, minus " +
            "the HTTP response header. We will set this as the text of " +
            "the first paragraph in the document.\n\n" + content);
        

        //Replace the first paragraph text with the retrieved
        //file content. This operation is a little
        //complicated to understand. There is another sample script
        //that includes this operation and provides some more
        //comments. See that script for more information.
        var pgf = doc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;
        var textRange = new TextRange();
        
        textRange.beg.obj = textRange.end.obj = pgf;
        textRange.beg.offset = 0;
        textRange.end.offset = Constants.FV_OBJ_END_OFFSET;
        doc.TextSelection = textRange;
        textRange = doc.TextSelection;
        textRange.end.offset--;
        doc.TextSelection = textRange;
        doc.Clear(0);
        
        doc.AddText(textRange.beg, content);
        
        alert("The text:\n\n" + content + 
            "\n\n...was set as the first paragraph text in the active document. "
                + "We're done!");
    }
    
    else alert("A problem occurred while attempting to " +
        "open a socket connection to www.weststreetconsulting.com " +
        "on port 80. Make sure that you have internet connectivity " +
        "to this site and that there is no firewall blocking communications.");
        
}

//Else, if we did not find an active document, land 
//never attempted the socket connection, let's warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");
