﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (any)

DESCRIPTION:

This script creates a "Graphic Utilities" palette window.
It is an example of how to create a modeless dialog box
and respond to the the controls on it.
The main point is to show you how to create a dialog box
and actually make it do something useful.
This is a longer script with various amounts of complexity.
Expect that it will take some time to understand all parts of it.

Ideally, you would probably want to hook the 
DoGraphicUtilitiesDialog() function into a menu command, so you
don't have to run the script every time you want to produce the 
dialog box. There is a sample that shows how to create
menus and commands.... with some study, you
should be able to figure out how to hook them together.

*/

DoGraphicUtilitiesDialog();

function DoGraphicUtilitiesDialog()
{
    //To prevent duplicate instances of the palette, let's make sure it doesn't
    //exist already
    var graphicUtilsDlg = Window.find ("palette", "My Graphic Utilities");
    if(graphicUtilsDlg != null) return;
    
    //This is an array of choices that will appear in a drop-down
    //list. It represents the available amounts to increment or
    //decrement the dimensions of a graphic with associated buttons,
    //in centimeters. It is defined here for convenience. To change the
    //options that appear in the dropdown, simply change these values.
    //You can add or remove from the list as well.
    var incrementChoices = [0.1, 0.2, 0.25, 0.5, 1.0, 2.0, 3.0]
    
    //Use the Window class to create the main palette. The "palette" argument
    //makes it a modeless dialog, The "My Graphic Utilities" argument defines
    //the title for the dialog. [100, 100, 340, 260] are coordinates that define
    //the initial position and size of the dialog, where the numbers (in pixels) are:
    //
    // [upperLeftX, upper leftY, lowerRightX, lowerRightY]
    //
    //...so the resulting affect is that the top left of the palette will appear at
    //at coordinates 100, 100, where 0,0 is the top left of the screen, and the
    //bottom right corner is at 340, 160. With that, the palette will be 240 pixels
    //wide and 160 pixels tall.
    //
    //When the controls are added to the palette, they all have coordinates that
    //operate in a similar manner, except that the 0,0 reference point changes
    //to the upper left corner of the palette, etc. These types of windows
    //also have a comprehensive feature set that allows automatic resizing
    //and alignment, but for simplicity, everything in this sample is a fixed
    //position and size.
    //
    //NOTE: There are alternative methods for creating palettes, including methods
    //where you don't explicitly set dimensions and coordinates. Rather, you set it
    //up so that things "float" naturally with reference to each other and the overall
    //size of the palette. It's a little more complicated but often produces superior output.
    //In this case, we'll use strict coordinates to keep it more understandable.
    graphicUtilsDlg = new Window("palette", "My Graphic Utilities", [100, 100, 340, 260]);  
    
    //Let's add a panel to contain most of the controls. Windows can contain panels,
    //then panels can contain controls. It's a way of grouping things.
    graphicUtilsDlg.toolsPanel = graphicUtilsDlg.add("panel", [10, 10, 230, 130], "Tools");

    //Add a little static text. Note how the identification of the parent object is
    //now graphicUtilsDlg.toolsPanel, showing the hierarchy of object ownership.
    graphicUtilsDlg.toolsPanel.add("staticText", [10, 11, 125, 26], "Size graphic");

    //Add the drop-down list to the panel and configure the selection list
    //with the incrementChoices defined earlier.
    graphicUtilsDlg.toolsPanel.incrementDropDown = 
      graphicUtilsDlg.toolsPanel.add("dropdownlist", [10, 30, 60, 50], incrementChoices);
    graphicUtilsDlg.toolsPanel.incrementDropDown.selection =
      graphicUtilsDlg.toolsPanel.incrementDropDown.items[0];
      
    //Add another bit of static text, a units label for the dropdown.    
    graphicUtilsDlg.toolsPanel.add("staticText", [65, 35, 85, 50], "cm.");

    graphicUtilsDlg.toolsPanel.heightIncreaseButton = 
      graphicUtilsDlg.toolsPanel.add("button", [90, 30, 150, 50], "Height +");

    //Add the four buttons to increase and decrease height and width.
    graphicUtilsDlg.toolsPanel.widthIncreaseButton = 
      graphicUtilsDlg.toolsPanel.add("button", [151, 30, 211, 50], "Width +");

    graphicUtilsDlg.toolsPanel.heightDecreaseButton = 
      graphicUtilsDlg.toolsPanel.add("button", [90, 52, 150, 72], "Height -");

    graphicUtilsDlg.toolsPanel.widthDecreaseButton = 
      graphicUtilsDlg.toolsPanel.add("button", [151, 52, 211, 72], "Width -");
    
    //Add the shrinkwrap and size-to-fit buttons
    graphicUtilsDlg.toolsPanel.shrinkwrapButton = 
      graphicUtilsDlg.toolsPanel.add("button", [10, 85, 80, 105], "Shrinkwrap");

    graphicUtilsDlg.toolsPanel.sizeToFitButton = 
      graphicUtilsDlg.toolsPanel.add("button", [85, 85, 155, 105], "Size To Fit");

    //Add the close button. Note we are adding this directly to the window object,
    //so the panel will not own it.
    graphicUtilsDlg.closeBtn = graphicUtilsDlg.add("button", [170, 135, 230, 155], "Close");

    //Now begins a set of "callbacks", that is, functions that execute when
    //controls are operated. For example, this first one defines what to do
    //when somebody clicks the "heightIncreaseButton". There are a wide
    //variety of user actions that you can respond to... in this case, we only
    //care about a click. Because we have not specified any code for any of
    //the other types of actions (callbacks), nothing happens when those
    //actions are performed.
    graphicUtilsDlg.toolsPanel.heightIncreaseButton.onClick = function() 
    {
        //Retrieve the increment amount from the dropdown, that is, the current dropdown 
        //selection. You have to retrieve a dropdown selection as a string, so
        //the parseFloat() is required to convert it into a real number.
        var increment = parseFloat(graphicUtilsDlg.toolsPanel.incrementDropDown.selection.text);
        if(increment <= 0)  increment = 1;
        
        //...then, call our custom function for resizing a graphic object. Note that the
        //third argument is zero, because that reflects the amount to increase
        //the width, and we don't want to change the width. Also, the last argument
        //is true which means we want to expand, versus shrink, the object.
        SizeGraphic(null, increment, 0, true);
	};

    //Callback for clicking the Height decrease button, similar logic.
    graphicUtilsDlg.toolsPanel.heightDecreaseButton.onClick = function() 
    {
        var increment = parseFloat(graphicUtilsDlg.toolsPanel.incrementDropDown.selection.text);
        if(increment <= 0)  increment = 1;
        SizeGraphic(null, increment, 0, false);
	};

    //Callback for clicking the Width increase button, similar logic.
    graphicUtilsDlg.toolsPanel.widthIncreaseButton.onClick = function() 
    {
        var increment = parseFloat(graphicUtilsDlg.toolsPanel.incrementDropDown.selection.text);
        if(increment <= 0)  increment = 1;
        SizeGraphic(null, 0, increment, true);
	};

    //Callback for clicking the Width decrease button, similar logic.
    graphicUtilsDlg.toolsPanel.widthDecreaseButton.onClick = function() 
    {
        var increment = parseFloat(graphicUtilsDlg.toolsPanel.incrementDropDown.selection.text);
        if(increment <= 0)  increment = 1;
        SizeGraphic(null, 0, increment, false);
    };

    //Callback for clicking the Shrinkwrap button. We just call our custom
    //ShrinkwrapGraphic function with a fixed buffer of 0.1 cm. We could
    //use the value from the dropdown as well if we wanted to.
    graphicUtilsDlg.toolsPanel.shrinkwrapButton.onClick = function() 
    {
        ShrinkwrapGraphic(null, null, .1);
	};

    //Callback for clicking the Size To Fit button. We just call our custom
    //SizeToFitGraphic function with a fixed buffer of 0.1 cm. 
    graphicUtilsDlg.toolsPanel.sizeToFitButton.onClick = function() 
    {
        SizeToFitGraphic(null, null, .1);
	};

    //Callback for clicking the Close button, just want to close the palette.
    graphicUtilsDlg.closeBtn.onClick = function() 
    {
        graphicUtilsDlg.close();
        graphicUtilsDlg = null;
	};

    //This gets called by close() above and when
    //you click X in the upper right corner.
    graphicUtilsDlg.onClose = function() 
    {
        graphicUtilsDlg = null;
	};

    //Finally, after all is configured and defined, make the palette appear 
    //on screen.
    graphicUtilsDlg.show();
    
 }




//****************************************
// FUNCTIONS

//This function increase or decreases the height and/or width of
//a graphic object,. Arguments are:

// - graphic - A graphic or anchored frame object. You can also send
//                     null to assume the currently-selected object, if any.
// - height_inc - The amount of centimeters to increase or decrease
//                          the height. Send zero to leave the height as-is
// - width_inc - The amount of centimeters to increase or decrease
//                          the width. Send zero to leave the width as-is
// - expand - Flag that indicates whether you want to expand or shrink
//                     the object by height_inc and width_inc. Send true to do
//                     an expansion, false to do a shrink
function SizeGraphic(graphic, height_inc, width_inc, expand)
{
    //If null was sent for the graphic, attempt to derive it from the current
    //selection in the active document. If we still can't get a graphic object,
    //exit the function.
    if(graphic == null)
    {
        var doc = app.ActiveDoc;
        if(doc.ObjectValid()) graphic = doc.FirstSelectedGraphicInDoc;
        if(!graphic.ObjectValid()) return;
    }

    //if we are expanding the object, add the increment amounts to
    //the height and width, respectively. To convert centimeters
    //to the point value that these properties use, we have to multiply
    //by 1857710 (that is, 1857710 points = 1 cm).
    if(expand)
    {
        graphic.Width = graphic.Width + (width_inc * 1857710);
        graphic.Height = graphic.Height + (height_inc * 1857710);
    }
    
    //if shrinking the object, similar concept, except subtract the values
   else
    {
        graphic.Width = graphic.Width - (width_inc * 1857710);
        graphic.Height = graphic.Height - (height_inc * 1857710);
    }
}

//This function shrinkwraps an anchored frame around a graphic
//leaving the amount of space specified by 'buffer' (in cm) on all
//sides. The buffer can be zero. If null is sent for the anchored
//frame and/or the graphic, the current selection is used, if
//any
function ShrinkwrapGraphic(aframe, graphic, buffer)
{
    //if the aframe and/or graphic were sent here as null, 
    //use these custom functions to attempt to derive from
    //the current selection. In the end, if we don't have both,
    //we'll exit the function
    if(aframe == null) aframe = GetSelectedAFrame (null);
    if(graphic == null) graphic = GetSelectedGraphic(null);
    
    if(graphic == null || aframe == null) return;
    
    //First, let's size the anchored frame. We'll set the height and width
    //to the same as the graphic, adding additional space for the specified
    //buffer. We multiply the buffer times 1857710 to convert cm to
    //points, which is the unit for these properties. Also, we multiply
    //by 2 because we need the buffer on both sides (top and bottom, 
    //left and right)
    aframe.Width = graphic.Width + (2 * buffer * 1857710);
    aframe.Height = graphic.Height + (2 * buffer * 1857710);
    
    //Now we need to position the graphic, because although the
    //anchored frame is the right size, the graphic could be floating
    //around anywhere in there. The position of a graphic is denoted
    //by the LocX and LocY properties, which are simple x/y coordinates
    //that represent the respective offsets from the bottom left corner of the
    //frame. So, it's simple... we want the coordinates to be 0,0 to align it exactly
    //in the corner, plus any offset specified by the buffer to create the offset effect
    //on all sides.
    graphic.LocX = buffer * 1857710;
    graphic.LocY = buffer * 1857710;
}

//This function sizes a graphic to fit exactly within its anchored frame,
//without changing any of the dimensions of the frame.
//It preserves the aspect ratio of the graphic, so if the graphic
//and frame have different aspect ratios, there will be empty
//space on the top or right after the operation, depending on 
//which boundary was reached first.
function SizeToFitGraphic(aframe, graphic, buffer)
{
    //If null was sent for the graphic and/or aframe, 
    //attempt to derive them from the current
    //selection in the active document. If we still can't get the objects,
    //exit the function.
    if(aframe == null) aframe = GetSelectedAFrame (null);
    if(graphic == null) graphic = GetSelectedGraphic(null);
    
    if(graphic == null || aframe == null) return;
    
    //Let's get the aspect ratios of the graphic and the frame. These
    //will be used for calculations and will not change throughout the 
    //operations
    var aframeAspectRatio = aframe.Height / aframe.Width;
    var graphicAspectRatio = graphic.Height / graphic.Width;
    
    //If the height/width aspect ratio of the frame is less than the 
    //graphic, the maximum height of the graphic will be reached
    //before the maximum width. That is, if the graphic were
    //stretched manually, the height would reach the anchored frame
    //height before the width. So, we will end up with some empty space 
    //on the right after it is over.
    if(aframeAspectRatio < graphicAspectRatio)
    {
        //We know the height is going to max out first, so we can go ahead
        //and set it to the height of the frame, minus any buffer that
        //was specified.
        graphic.Height = aframe.Height - (2 * buffer * 1857710);
        
        //Now, just a little math. To preserve the aspect ratio based 
        //on the new height, we can divide the new height
        //by the aspect ratio to get the required new width.
        graphic.Width = graphic.Height / graphicAspectRatio;
    }

    //Same concept if the graphic aspect ratio is less than the
    //frame aspect ratio, except we will do the opposite.... set 
    //the width based on the frame width and then a little
    //math to derive the proper height to ensure the same
    //aspect ratio.
    else 
    {
        graphic.Width = aframe.Width - (2 * buffer * 1857710);
        graphic.Height = graphicAspectRatio  * graphic.Width;
    }
    
    //Now just like shrinkwrapping, we have the graphic at the size we 
    //want, but it could be floating around anywhere. Let's position
    //it properly at the 0,0 coordinate, plus any offset required by 
    //the specified buffer
    graphic.LocX = buffer * 1857710;
    graphic.LocY = buffer * 1857710;
}

//This function retrieves the currently selected
//graphic object. If an anchored frame is selected,
//it returns the first graphic it finds in the frame,
//if any. If nothing is ever found, it returns null.
function GetSelectedGraphic(doc)
{
    //If null was sent for the document, assume the
    //active document. If there is no active document,
    //exit the function.
    if(doc == null) doc = app.ActiveDoc;
    if(doc == null) return null;
   
    //Get the first selected graphic in the document    
    graphic = doc.FirstSelectedGraphicInDoc;
   
    //If we don't have anything selected, it's 
    //automatically a bust.
    if(!graphic.ObjectValid()) graphic = null;
    
    //If we had an anchored frame selected, let's
    //look deeper to find the first graphic in the frame.
    else if(graphic.type == Constants.FO_AFrame)
    {
        graphic = graphic.FirstGraphicInFrame;
        if(!graphic.ObjectValid()) graphic = null;
    }
    
    //Return whatever we have.
    return graphic;
}

//This function retrieves the currently-selected
//anchored frame, or the anchored frame
//that contains the currently-selected graphic.
//If no frame can be found, it returns null.
function GetSelectedAFrame(doc)
{
    //If null was sent for the document, assume the
    //active document. If there is no active document,
    //exit the function.
    if(doc == null) doc = app.ActiveDoc;
    if(doc == null) return null;
   
    //Get the first selected graphic in the document.
    graphic = doc.FirstSelectedGraphicInDoc;
   
    //If nothing is selected, this is automatically a bust.    
    if(!graphic.ObjectValid()) graphic = null;
   
    //If we do have something selected but it is not an 
    //anchored frame, let's walk up to the parent frame
    //and see if it is an anchored frame. If
    else if(graphic.type != Constants.FO_AFrame)
    {
        graphic = graphic.FrameParent;
        
        //If we didn't find any parent or the parent is not
        //an anchored frame, reset the return variable to null
        if(!graphic.ObjectValid()) 
            graphic = null;
        else if(graphic.type != Constants.FO_AFrame)
            graphic = null;
    }
    
    //Return whatever we've got.
    return graphic;
}




