﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  Any book or document.

DESCRIPTION:
    
This is a script that will step through and do a basic
test of every Hypertext marker URL in a book or document.
When processing a book, only currently-open documents
are processed.

For each URL, the script attempts a basic connection
and reads the HTTP response code. Any attempts
that return a code of 200 OK are considered OK.
Any that return a code of 404 Not Found are considered
broken. Any others are considered questionable.

The script uses the built-in "book error log" to report
results. It is not an attractive report, but it is readily
available for use and much easier than building
your own report. You could build your own report
with any nature of beautiful formatting if you wanted,
but that is beyond the scope of this free script.

For a marker to be tested, it must be the type
of Hypertext, the text must start with "message URL",
and the text must not contain "mailto::". There may
be other contingencies that have not been considered
yet and these could be easily built into the script.

Comments in this script are brief! For the socket
activity specifically, see the more basic
networking sample if you want more details
about what is going on.

*/

checkHypertextLinks_Main();


function checkHypertextLinks_Main()
{
    //Get the active book or document. We can
    //process either.
    var doc = app.ActiveDoc;
    var book = app.ActiveBook;
    
    //If we have a document, confirm and go    
    if(doc.ObjectValid()) 
    {
        if(confirm("Do you want to run the hypertext " 
            + "link checker on the following active "
            + "document?\n\n" + doc.Name))
        {
            checkHypertextLinks_Doc(doc, false);
        }
    }
    
    //If instead we have a book, confirm and go
    else if(book.ObjectValid())
    {
        if(confirm("Do you want to run the hypertext " 
            + "link checker on the following active "
            + "book? Any chapter files "
            + "that are not open will be skipped.\n\n" + book.Name))
        {
            checkHypertextLinks_Book(book);
        }
    }
    
    //If we have nothing, alert and abort.
    else
    {
        alert("No active book or document. Cannot continue.");
        return;
    }
    
    //Finish up the report.
    callErrorLog(null, null, null, "-----------------------------------------------");
    callErrorLog(null, null, null, "-----------------------------------------------");
    callErrorLog(null, null, null, "Processing complete.");
    callErrorLog(null, null, null, "Note that some non-OK HTTP responses, especially "
        + "those in the 300-series, may or may not represent a broken link. "
        + "Different internet hosts implement the HTTP standard in different ways. "
        + "For any link that is shown as a possible issue, you will need to manually "
        + "investigate.");
    callErrorLog(null, null, null, "-----------------------------------------------");
    
    alert("Script complete.");
}


function checkHypertextLinks_Book(book)
{
    //Initialize the report
    callErrorLog(null, null, null, "-----------------------------------------------");
    callErrorLog(null, null, null, "-----------------------------------------------");
    callErrorLog(null, null, null, "HYPERTEXT LINK CHECK FOR: ");
    callErrorLog(null, null, null, book.Name);
    callErrorLog(null, null, null, "-----------------------------------------------");
    callErrorLog(null, null, null, "-----------------------------------------------");

    //Iterate through all the components and look for 
    //associated documents. For any document that
    //can be found, send it to the link checker.
    //Any that can't be, warn and move on.
    var comp = book.FirstComponentInBook;
    while(comp.ObjectValid())
    {
        //We only care about components that represent files. We don't
        //care about folders or groups.
        if(comp.ComponentType == Constants.FV_BK_FILE)
        {
            //Use this locally-defined function to look for a corresponding
            //open document.
            var doc = getOpenFile(comp.Name, false, false, false);
            
            //If we found an open document for the component, do the deed.
            if(doc != null && doc.ObjectValid()) 
            {
                checkHypertextLinks_Doc(doc, true);
            }
            
            //Otherwise, report that we couldn't find it.
            else
            {
                callErrorLog(null, null, null, "The file, \""
                    + comp.Name 
                    + "\" is not currently open and could not be processed.");
            }
        }
    
        //Get the next component
        comp = comp.NextBookComponentInDFSOrder;
    }
}


function checkHypertextLinks_Doc(doc, doingABook)
{
    //If we are not doing a whole book, let's initialize the report.
    //For a book, this is already done.
    if(!doingABook)
    {
        callErrorLog(null, null, null, "-----------------------------------------------");
        callErrorLog(null, null, null, "-----------------------------------------------");
        callErrorLog(null, null, null, "HYPERTEXT LINK CHECK FOR: ");
        callErrorLog(null, null, null, doc.Name);
        callErrorLog(null, null, null, "-----------------------------------------------");
        callErrorLog(null, null, null, "-----------------------------------------------");
    }
    //If we are doing a book, let's put in our file delimiter
    else
    {
        callErrorLog(null, null, null, "--------------------------------------");
        callErrorLog(null, doc, null, "--> Processing: " + doc.Name);
    }
    
    //Initialize some variables
    var markersProcessed = 0;
    var markersSkipped = 0;
    var marker = doc.FirstMarkerInDoc;
    
    //Begin the iteration through all markers in the document.
    while(marker.ObjectValid())
    {
        //Let's get the marker type and its text. we only
        //want Hypertext markers that contain the text
        //"message URL"
        var markerType = marker.MarkerTypeId.Name;
        var markerText = marker.MarkerText;
        if(markerType == "Hypertext" && 
            markerText.toLowerCase().indexOf("message url") == 0 &&
            markerText.toLowerCase().indexOf("mailto:") < 0)
        {
            //Let's get rid of the "message URL" part
            var url = markerText.substring(12, markerText.length);
            
            //Use this locally-defined function to remove any leading
            //and trailing whitespace.
            url = trim(trim(url));
            
            //Call the URL checker to get the first line of the 
            //response header
            var response = checkUrl(url);
            
            //Now, let's take a look at this response and prepare
            //an entry for the log.
            var logEntry;
            
            //A 200 OK is very good.
            if(response.toLowerCase().indexOf("200 ok") > -1)
            {
                logEntry = "Link OK: " + url;
            }
        
            //A 404 Not Found is definitely not good
            else if(response.toLowerCase().indexOf("404 not") > -1)
            {
                logEntry = "**BROKEN LINK**: " + url + " (" + response + ")";
            }
        
            //Anything else... hard to say. Let's just report what we received.
            else
            {
                logEntry = "**POSSIBLE ISSUE**: " + url 
                    + " - HTTP response: " + response;
            }
            
            //Write the log entry
            callErrorLog(null, doc, marker, logEntry);
            
            //Increment the counter of markers processed.
            markersProcessed++;
        }
    
        //If we are not processing this marker, increment the 
        //counter of markers skipped.
        else markersSkipped++;
        
        //...and finally, get the next marker in the document.
        marker = marker.NextMarkerInDoc;
    }
    
    //We are done with the document. Report our counters.
    callErrorLog(null, null, null, "Total hypertext markers checked: " 
        + markersProcessed);
    callErrorLog(null, null, null, "Total non-URL markers skipped: " 
        + markersSkipped);
}


//This is a very basic networking function
//that attempts to request a web page
//and get the first line of the response header.
//It is the responsibility of the code that calls
//this function to parse the header. The
//HTTP response code should be in there at 
//the end. A successful page retrieval would
//send back something like:
//
// HTTP/1.1 200 OK
function checkUrl(url)
{
    //The socket object doesn't like the http prefix
    //for site names. Let's strip it if present.
    url = url.replace("http://", "");
    url = url.replace("https://", "");
    
    //We need to know the base site name, and
    //the specific destiination page, if any.
    //If there is a forward slash delimiter in the URL,
    //we'll assume that anything before the first
    //one is the site, and anything after is the specific page.
    if(url.indexOf("/") > -1)
    {
        var site = url.substring(0, url.indexOf("/"));
        var page = url.substring(site.length, url.length);
    }
    //Otherwise, we just have a basic site name (e.g. www.google.com)
    //that goes to the default page.
    else
    {
        var site = url;
        var page = "/";
    }

    //Use the socket object to open a connection to the site.
    var conn = new Socket;
    if (conn.open (site + ":80", "UTF-8"))
    {
        //If we get here, we are at least able to talk to the server.
        //Let's ask for the page.
        var getRequest = "GET " + page + " HTTP/1.1\r\n" +
            "Host: " + site + "\r\n\r\n";
        
        //This setting is required.
        conn.encoding = "BINARY";
        
        //Submit the request
        conn.write(getRequest);
        
        //Now, let's read character-by-character until
        //we get to the first line break. That should
        //get us up to the response code.
        var endlessLoopCounter = 0;
        var response = "";
        var nextChar = "";
        while(nextChar != "\r" && nextChar != "\n")
        {
            //Add the current character to the response
            response += nextChar;
            
            //Read in a single character and add it to our running string
            nextChar = conn.read(1);
            
            //Check for an endless loop. We'll allow 100 characters to
            //go by before we decide there is a critical problem.
            if(endlessLoopCounter++ > 100)
            {
                //break out of the while loop.
                response = "UNKNOWN ERROR";
                break;
            }
        }
         
        //Close the connection 
        conn.close();
    }
    
    //If we could not even reach the server, let's just set the 404 code
    else response = "HTTP/1.1 404 Not Found";
    
    //Return the response
    return response;
}






//A shortcut function to find an open document or book based on the
//filepath. Arguments are:
//
// path (String) - The path of the file, using backslashes for delimiters. Case-insensitive.
// considerSubstring (Boolean) - If true, allows the path argument to be a substring
//                of the actual file path; for example, just the filename.
// considerBooks (Boolean) - If true, allows a search for a book based on path. Otherwise,
//                only documents are considered.
// considerHiddenFiles - If true, allows the consideration of hidden files (doc.IsOnScreen = false).
function getOpenFile(path, considerSubstring, considerBooks, considerHiddenFiles)
{
    var returnFile= null;
    var filePath;
    
    path = path.toLowerCase();
    
    var doc = app.FirstOpenDoc;
    while(doc != null && doc.ObjectValid())
    {
        if(considerHiddenFiles || doc.IsOnScreen)
        {
            filePath = doc.Name.toLowerCase();
            
            if(considerSubstring && filePath.indexOf(path) > -1)
                returnFile = doc;
            else if(filePath == path)
                returnFile = doc;
        }
            
        if(returnFile != null) doc = null;
        else doc = doc.NextOpenDocInSession;
    }
    
    if(considerBooks && returnFile == null) 
    {
        var book = app.FirstOpenBook;
        while(book.ObjectValid())
        {
            if(considerHiddenFiles || book.IsOnScreen)
            {
                filePath =book.Name.toLowerCase();
                
                if(considerSubstring && filePath.indexOf(path) > -1)
                    returnFile = book;
                else if(filePath == path)
                    returnFile = book;
            }
                
            if(returnFile != null) book = null;
            else book = book.NextOpenBookInSession;
        }
    }
    
    return returnFile;
}



// Creates an entry in the built-in "book error log",
// the same log produced during book updates.
// FM provides access to this log as a convenience.
// Send it as much info as you have, with null
// for anything you don't. If there is a valid doc or
// book object, a hyperlink will be created to the file.
// If their is a valid "object" object which could be
// a marker, element, cross-reference, etc.,
// the report should show a hyperlink directly to it.
// If you want just text, just send text with null for
// everything else.
// The syntax for the call to the log is a bit
// complicated and is described in the FDK user
// documentation. The log is actually driven by
// an FDK client, which this function calls to do the
// work. See the FDK documentation for more info.
function callErrorLog(book, doc, object, text)
{
    var arg;
    
    arg = "log ";
    
    if(book == null || book == 0 || !book.ObjectValid())
        arg += "-b=0 ";
    else arg += "-b=" + book.id + " ";
    if(doc == null || doc == 0 || !doc.ObjectValid())
        arg += "-d=0 ";
    else arg += "-d=" + doc.id + " ";
    if(object == null || object == 0 || !object.ObjectValid())
        arg += "-o=0 ";
    else arg += "-o=" + object.id + " ";
    
    if(text != null) arg += "--" + text;

    CallClient("BookErrorLog", arg);
}

// A shortcut function to strip leading and trailing
// whitespace from a string
function trim(str)
{
    while(str.indexOf(" ") == 0 && str.length > 0)
        str = str.substring(1, str.length);
    while(str.lastIndexOf(" ") == (str.length - 1) && str.length > 0)
        str = str.substring(0, str.length - 1);
        
    return str;
}    
    
    

